/* 
   DOCoder.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: DOCoder.m,v 1.5 1999/01/21 13:31:59 helge Exp $

#import "common.h"
#import "DOCoder.h"
#import "NGConnection+private.h"

@implementation DOCoder

+ (id)coderWithStream:(id<NGSerializer,NGStream>)_stream
  connection:(NGConnection *)_c {
  
  return AUTORELEASE([[self alloc] initWithStream:_stream connection:_c]);
}

- (id)initWithStream:(id<NGSerializer,NGStream>)_stream
  connection:(NGConnection *)_c {
  
  if ((self = [super initWithStream:_stream])) {
    self->connection = RETAIN(_c);

    self->classForCoder      = @selector(classForDOCoder:);
    self->replObjectForCoder = @selector(replacementObjectForDOCoder:);
  }
  return self;
}
- (id)initWithStream:(id<NGSerializer,NGStream>)_stream {
  return [self initWithStream:_stream connection:nil];
}

// accessors

- (NGConnection *)connection {
  return self->connection;
}

- (BOOL)isValid {
  return (self->connection != nil) ? YES : NO;
}

// copy state

- (BOOL)isBycopy { // encode object
  return NO;
}
- (BOOL)isByref { // vend proxy
  return YES;
}

// references

- (void)invalidate { // called when the connection is deallocated
  self->connection = nil;
}

// operations

- (void)encodeObject:(id<NSObject>)_object bycopy:(BOOL)_flag {
  [self encodeObject:_object];
}
- (void)encodeValueOfObjCType:(const char *)_type at:(void *)_v bycopy:(BOOL)_flag {
  [self encodeValueOfObjCType:_type at:_v];
}

// invocations

- (BOOL)isInvocationInMode {
  return self->isRequest;
}
- (void)encodeInvocation:(NSInvocation *)_invocation {
  self->isRequest = YES;
  [self encodeRootObject:_invocation];
}
- (void)encodeInvocationResult:(NSInvocation *)_invocation {
  self->isRequest = NO;
  [self encodeRootObject:_invocation];
}

@end

@interface NSObject(ClassForCoder_Colon)

- (Class)classForCoder:(NSCoder *)_coder;

@end

@implementation NSObject(DOCoder)

- (Class)classForDOCoder:(DOCoder *)_coder {
  NSLog(@"do: called classForDOCoder: on 0x%08X<%s>..",
        self, class_get_class_name(*(Class *)self));
  
  if ([self respondsToSelector:@selector(classForCoder:)])
    return [self classForCoder:_coder];
  else
    return [self classForCoder];
}

- (id)replacementObjectForDOCoder:(DOCoder *)_coder {
  NSLog(@"do: called replacementObjectForDOCoder: on 0x%08X<%s> ..",
        self, class_get_class_name(*(Class *)self));
  
  if ([_coder isBycopy]) {
    NSLog(@"do: is bycopy ..");
    return self;
  }
  else {
    id replacement = nil;
    NGConnection *connection = [_coder connection];

    NSAssert(connection, @"no connection set ..");
    replacement = [connection vendObject:self]; // encode local-proxy
    NSLog(@"do: replacement for 0x%08X<%s> is 0x%08X<%s>",
           self, class_get_class_name(*(Class *)self),
           replacement, class_get_class_name(*(Class *)self));

    return replacement;
  }
}

@end

@implementation NSNumber(DOCoder)

- (id)replacementObjectForDOCoder:(DOCoder *)_coder {
  return self; // always bycopy
}

@end

@implementation NSString(DOCoder)

- (Class)classForDOCoder:(DOCoder *)_coder {
  return [self classForCoder];
}

- (id)replacementObjectForDOCoder:(DOCoder *)_coder {
  return self; // always bycopy
}
@end

@implementation NSMutableString(DOCoder)

- (Class)classForDOCoder:(DOCoder *)_coder {
  return [self classForCoder];
}

- (id)replacementObjectForDOCoder:(DOCoder *)_coder {
  if ([_coder isBycopy])
    return self;
  else
    return [[_coder connection] vendObject:self]; // encode reference
}

@end

@implementation NGInternetSocketAddress(DOCoder)
- (id)replacementObjectForDOCoder:(DOCoder *)_coder {
  return self; // always bycopy
}
@end

@implementation NGLocalSocketAddress(DOCoder)
- (id)replacementObjectForDOCoder:(DOCoder *)_coder {
  return self; // always bycopy
}
@end

@implementation NSMethodSignature(DOCoder)

- (id)replacementObjectForDOCoder:(DOCoder *)_coder {
  return self; // always bycopy
}

- (void)encodeWithCoder:(NSCoder *)_coder {
  const char *value = [[self objCTypes] cString];
  
  NSLog(@"do: WARNING: encoding method signature %@", self);
  [_coder encodeValueOfObjCType:@encode(char *) at:&value];
}

- (id)initWithCoder:(NSCoder *)_coder {
  char *value = NULL;
  
  RELEASE(self);
  [_coder decodeValueOfObjCType:@encode(char *) at:&value];
  self = [NSMethodSignature signatureWithObjCTypes:value];
  NGFree(value); value = NULL;
  
  return RETAIN(self);
}

@end
