/* 
   DOProtocol.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: DOProtocol.m,v 1.2 1998/10/02 10:41:46 helge Exp $

#import "common.h"
#import "DOProtocol.h"
#import "NGConnection+private.h"
#import "DOCoder.h"
#import "NSInvocation+DO.h"

static unsigned sendSerial = 0;

unsigned DOSendRequestHead(id<NGSerializer> _destination, DOMessageCode _code) {
  sendSerial++;

  NSCAssert((_code & REPLY) == 0, @"used request-send for reply ..");
  
  // send request head
  [_destination serializeChar:_code];
  [_destination serializeInt:sendSerial];

  return sendSerial;
}

void DOSendReplyHead(id<NGSerializer> _src, DOMessageCode _code, unsigned _serial) {
  NSCAssert(_code & REPLY, @"used reply-send for request ..");
  
  // send request head
  [_src serializeChar:_code];
  [_src serializeInt:_serial];
}

DOMessageCode DOReceiveHead(id<NGSerializer> _src, unsigned *serial_) {
  char code;
  code     = [_src deserializeChar];
  *serial_ = [_src deserializeInt];
  return code;
}

char *DOGetSignature(id<NGSerializer> _send, id<NGSerializer> _rcv,
                     SEL _selector, unsigned _remoteId) {
  char          *name = _selector ? (char *)sel_get_name(_selector) : NULL;
  DOMessageCode code;
  unsigned      serial, replySerial;

  serial = DOSendRequestHead(_send, DOMsg_SIGNATURE);
  [_send serializeDataAt:&name      ofObjCType:@encode(char *) context:nil];
  [_send serializeDataAt:&_remoteId ofObjCType:@encode(int)    context:nil];
  [(id)_send flush];

  code = DOReceiveHead(_rcv, &replySerial);
  NSCAssert(code & REPLY,          @"got a request, expected reply ..");
  NSCAssert(serial == replySerial, @"unorded messages ..");
  
  [_rcv deserializeDataAt:&name ofObjCType:@encode(char *) context:nil];
  
  return name;
}
