/* 
   NGDistantObject.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGDistantObject.m,v 1.6 1999/01/21 13:31:59 helge Exp $

#import "common.h"
#import "NGDistantObject.h"
#import "NGConnection+private.h"
#import "DOCoder.h"

/*
  Uses debug-topic 'proxy'.
 */

@implementation NGDistantObject

+ (NGDistantObject *)proxyWithLocal:(id)_ref connection:(NGConnection *)_owner {
  return AUTORELEASE([[self alloc] initWithLocal:_ref connection:_owner]);
}
+ (NGDistantObject *)proxyWithTarget:(unsigned)_reference
  connection:(NGConnection *)_owner {

  return AUTORELEASE([[self alloc] initWithTarget:_reference connection:_owner]);
}

- (id)initWithLocal:(id)_reference connection:(NGConnection *)_owner {
  NSLog(@"proxy: create proxy for local 0x%08X ..", _reference);

  self->owner     = RETAIN(_owner);
  self->reference = (unsigned)_reference;
  self->selToSignature = NULL;

  return self;
}
- (id)initWithTarget:(unsigned)_reference connection:(NGConnection *)_owner {
  NSLog(@"proxy: create proxy for reference 0x%08X ..", _reference);
  
  self->owner     = RETAIN(_owner);
  self->reference = _reference;
  
  self->selToSignature =
    NSCreateMapTable(NSNonOwnedCStringMapKeyCallBacks,
                     NSObjectMapValueCallBacks,
                     19);

  return self;
}

- (void)gcFinalize {
  if (self->selToSignature) {
    [self->owner releaseProxy:self];
    NSLog(@"proxy: released remote-proxy 0x%08X for reference 0x%08X ..",
          self, self->reference);
  }
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  [self gcFinalize];

  RELEASE(self->owner); self->owner     = nil;

  if (self->selToSignature) {
    NSFreeMapTable(self->selToSignature);
    self->selToSignature = NULL;
  }
  [super dealloc];
}
#endif

// accessors

- (NGConnection *)connectionForProxy {
  return self->owner;
}

- (void)setProtocolForProxy:(Protocol *)_protocol {
  NSLog(@"not implemented ..");
  abort();
}

// forwarding

- (NSMethodSignature *)methodSignatureForSelector:(SEL)_selector {
  BOOL isLocal = self->selToSignature ? NO : YES;

  NSAssert(self->owner, @"no owner for proxy ..");

  if (isLocal) {
    struct objc_method* mth;

    NSLog(@"proxy: asked for signature of %@ in local proxy 0x%08X !",
           NSStringFromSelector(_selector), self);

    if (_selector)
	return nil;

    mth = class_get_instance_method(self->isa, _selector);
    return mth ? [NSMethodSignature signatureWithObjCTypes:mth->method_types]
		: nil;
    
    [[NGTool tool] prepareForAbort];
    [[NGTool tool] coreDump];
    return nil;
  }
  else {
    NSMethodSignature *signature = nil;
    const char        *selName   = sel_get_name(_selector);
    
    NSLog(@"proxy: proxy 0x%08X<%s> was asked for method signature of %s",
           self, class_get_class_name(*(Class *)self), selName);

    signature = NSMapGet(self->selToSignature, (void *)selName);
    if (signature == nil) {
      if (_selector->sel_types) {
        // got a typed selector
        
        NSLog(@"proxy got method types '%s' for selector '%@'",
              _selector->sel_types, NSStringFromSelector(_selector));
        
        signature = [NSMethodSignature signatureWithObjCTypes:_selector->sel_types];
      }
      else {
        // got an untyped selector
        
        signature = [self->owner signatureForSelector:_selector
                                 ofProxy:self];
      }
      if (signature)
        NSMapInsert(self->selToSignature, (void *)selName, (void *)signature);
    }
    return signature;
  }
}

- (void)forwardInvocation:(NSInvocation *)_invocation {
  BOOL isLocal = self->selToSignature ? NO : YES;
  
  NSAssert(self->owner, @"no owner for proxy ..");

  if (isLocal) {
    NSLog(@"proxy: send message %@ to local proxy 0x%08X !",
          NSStringFromSelector([_invocation selector]), self);
    abort();
  }
  else {
    NSLog(@"proxy 0x%08X was asked to perform %sinvocation with selector %@",
           self,
           [[_invocation methodSignature] isOneway] ? "oneway " : "",
           NSStringFromSelector([_invocation selector]));

    [self->owner performInvocation:_invocation forProxy:self];
  }
}

- (BOOL)respondsToSelector:(SEL)_selector {
  BOOL isLocal = self->selToSignature ? NO : YES;
  
  NSAssert(self->owner, @"no owner for proxy ..");
  
  if (isLocal) {
    NSLog(@"proxy: asked local proxy 0x%08X whether it responds to '%@' !",
          self, NSStringFromSelector(_selector));
    abort();
  }
  else {
    NSMethodSignature *signature = [self methodSignatureForSelector:_selector];
    
    NSLog(@"proxy: asked remote proxy 0x%08X whether it responds to '%@' !",
          self, NSStringFromSelector(_selector));

    return signature ? YES : NO;
  }
}

// description

/*
- (NSString *)description {
  BOOL isLocal = self->selToSignature ? NO : YES;
  
  NSAssert(self->owner, @"no owner for proxy ..");

  if (isLocal) {
    return [NSString stringWithFormat:@"<%s[0x%08X]: local>",
                       class_get_class_name(self->isa), self];
  }
  else {
    id inv = nil;
    
    inv = [NSInvocation invocationWithMethodSignature:
                          [self methodSignatureForSelector:_cmd]];
    [inv setTarget:self];
    [inv setSelector:_cmd];
    [self->owner performInvocation:inv forProxy:self];
    [inv getReturnValue:&inv];

    return [NSString stringWithFormat:@"<%s[0x%08X]: remote=%@>",
                       class_get_class_name(self->isa), self, inv];
  }
}
*/

// NSCoding

- (Class)classForCoder {
  return [NGDistantObject class];
}
- (Class)classForDOCoder:(DOCoder *)_coder {
  return [NGDistantObject class];
}

- (id)replacementObjectForDOCoder:(DOCoder *)_coder {
  return self;
}

- (void)encodeWithCoder:(NSCoder *)_encoder {
  BOOL isLocal = self->selToSignature ? NO : YES;

  [_encoder encodeValueOfObjCType:@encode(BOOL)     at:&isLocal];
  [_encoder encodeValueOfObjCType:@encode(unsigned) at:&(self->reference)];
}

- (id)initWithCoder:(NSCoder *)_decoder {
  BOOL     isLocal;
  unsigned ref;

  [_decoder decodeValueOfObjCType:@encode(BOOL)     at:&isLocal];
  [_decoder decodeValueOfObjCType:@encode(unsigned) at:&ref];

  if (isLocal) { // was local, now is remote
    self->owner     = RETAIN([(DOCoder *)_decoder connection]);
    self->reference = ref;

    if ([self->owner hasProxyForRemoteReference:self->reference]) {
      NGDistantObject *replacement =
        [self->owner proxyForRemoteReference:self->reference];

      NSLog(@"proxy: replaced proxy 0x%08X by existing proxy 0x%08X<%s>",
            self, ref, class_get_class_name(*(Class *)ref));
      
      RELEASE(self);
      return RETAIN(replacement);
    }
    else {
      NSLog(@"proxy: registering proxy 0x%08X for remote 0x%08X ..", self, ref);

      self->selToSignature =
        NSCreateMapTable(NSNonOwnedCStringMapKeyCallBacks,
                         NSObjectMapValueCallBacks,
                         19);
  
      [self->owner registerProxy:self];
      return self;
    }
  }
  else { // was remote, now is local
    NSLog(@"proxy: replaced proxy 0x%08X by local-object 0x%08X<%s>",
          self, ref, class_get_class_name(*(Class *)ref));
    
    RELEASE(self);
    return [(id)ref retain]; // retained, a bug ?
  }
  return self;
}

- (id)awakeAfterUsingCoder:(NSCoder *)_coder {
  return self;
}

@end
