/* 
   NSInvocation+DO.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NSInvocation+DO.m,v 1.2 1998/10/02 11:07:38 helge Exp $

#import "common.h"
#import "NSInvocation+DO.h"
#import "DOCoder.h"

@implementation NSInvocation(NSCoding)

- (Class)classForDOCoder:(DOCoder *)_coder {
  return [NSInvocation class];
}
- (id)replacementObjectForDOCoder:(DOCoder *)_coder {
  return self; // always bycopy
}

- (void)encodeWithCoder:(DOCoder *)_encoder {
  NSMethodSignature *sig = [self methodSignature];
  SEL               sel  = [self selector];
  char              isIn = [_encoder isInvocationInMode] ? 1 : 0;

  [_encoder encodeValueOfObjCType:@encode(char) at:&isIn];
  
  if (sig == nil)
    sig = [[self target] methodSignatureForSelector:sel];
  
  if (isIn) {
    const char *value;
    char       isOneway = [sig isOneway] ? 1 : 0;

    value = sel ? sel_get_name(sel) : NULL;
    [_encoder encodeValueOfObjCType:@encode(char *) at:&value];
    [_encoder encodeValueOfObjCType:@encode(char)   at:&isOneway];

    value = [[sig objCTypes] cString];
    [_encoder encodeValueOfObjCType:@encode(char *) at:&value];

    [_encoder encodeObject:[self target]];

    // in arguments
    {
      unsigned char argCount = [sig numberOfArguments] - 2;
      int i;
    
      //[_encoder encodeValueOfObjCType:@encode(unsigned char) at:&argCount];
      for (i = 0; i < argCount; i++) {
        BOOL isInArg = [sig isInArgumentAt:(i + 2)];
        
        if (isInArg) {
          BOOL           isBycopyArg;
          NSArgumentInfo argInfo;
          char           buffer[256];

          isBycopyArg = [sig isBycopyArgumentAt:(i + 2)];
          argInfo     = [sig argumentInfoAtIndex:(i + 2)];
          NSCAssert(argInfo.size < 255, @"unsupported argument size ..");
          [self getArgument:buffer atIndex:(i + 2)];
        
          [_encoder encodeValueOfObjCType:argInfo.type at:&buffer
                    bycopy:isBycopyArg];
        }
        else {
          ; // out argument ..
        }
      }
    }
  }
  else {
    const char *retType  = [sig methodReturnType];
    int        returnLen = (*retType != _C_VOID) ? [sig methodReturnLength] : 0;
    
    NSLog(@"invocation output mode ..");

    [_encoder encodeValueOfObjCType:@encode(int) at:&returnLen];
    if (returnLen > 0)
      [_encoder encodeValueOfObjCType:retType at:self->returnValue];
  }
}

- (id)initWithCoder:(DOCoder *)_decoder {
  NSMethodSignature *sig = nil;
  char isIn;

  [_decoder decodeValueOfObjCType:@encode(char) at:&isIn];
  
  if (isIn) {
    char *value = NULL;
    char isOneway;
    SEL  sel;
    id   object;

    RELEASE(self); self = nil;

    [_decoder decodeValueOfObjCType:@encode(char *) at:&value];
    sel = value ? sel_get_any_uid(value) : NULL;
    NGFree(value);

    [_decoder decodeValueOfObjCType:@encode(char) at:&isOneway];

    [_decoder decodeValueOfObjCType:@encode(char *) at:&value];
    sig = [NSMethodSignature signatureWithObjCTypes:value];
    NGFree(value); value = NULL;

    object = [_decoder decodeObject];

    self = [NSInvocation invocationWithMethodSignature:sig];
    [self setSelector:sel];
    [self setTarget:object];

    // in arguments
    {
      unsigned char argCount = [sig numberOfArguments] - 2;
      int i;
    
      //[_encoder encodeValueOfObjCType:@encode(unsigned char) at:&argCount];
      for (i = 0; i < argCount; i++) {
        BOOL isInArg = [sig isInArgumentAt:(i + 2)];
        
        if (isInArg) {
          NSArgumentInfo argInfo;
          char           buffer[256];

          argInfo     = [sig argumentInfoAtIndex:(i + 2)];
          NSCAssert(argInfo.size < 255, @"unsupported argument size ..");
          [_decoder decodeValueOfObjCType:argInfo.type at:&buffer];

          [self setArgument:buffer atIndex:(i + 2)];
        }
        else {
          ; // out argument ..
          NSLog(@"did not encode 'out' argument (sel=%@, idx=%i)",
                NSStringFromSelector(sel), i);
        }
      }
    }
    [self retainArguments];
    return RETAIN(self);
  }
  else {
    const char *retType;
    int returnLen;

    sig = [self methodSignature];
    retType = [sig methodReturnType];
    
    NSLog(@"invocation output mode ..");

    [_decoder decodeValueOfObjCType:@encode(int) at:&returnLen];
    NSAssert(returnLen == [sig methodReturnLength], @"invalid return size ..");

    if (returnLen > 0) {
      void *buffer = NGMalloc(returnLen);
      [_decoder decodeValueOfObjCType:retType at:buffer];
      [self setReturnValue:buffer];
    }
  }
  return nil;
}

@end

void __link_NSInvocation_DO(void) {
  __link_NSInvocation_DO();
}
