/* 
   NGNil.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGNil.m,v 1.3 1998/10/02 16:51:06 helge Exp $

#import "common.h"
#import "NGNil.h"

@implementation NGNil

static NGNil *nilObject = nil;

+ (id)new {
  return [self nilObject];
}
+ (id)nilObject {
  if (nilObject == nil) {
    nilObject = [[self alloc] init];
  }
  return nilObject;
}

+ (BOOL)isNil:(id)_obj {
  if (_obj == nil)       return YES;
  if (_obj == nilObject) return YES;
  return [_obj isNil];
}
- (BOOL)isNil {
  return YES;
}

- (BOOL)isEqual:(id)_object {
  if(_object == self) return YES;
  return [_object isKindOfClass:[NGNil class]];
}

// description

- (NSString *)description {
  return [self stringValue];
}

// values

+ (NSString *)stringValue {
  return @"Class<NSNil>";
}
- (NSString *)stringValue {
  return @"<nil>";
}

- (BOOL)boolValue {
  return NO;
}

- (char)charValue {
  return 0;
}
- (unsigned char)unsignedCharValue {
  return 0;
}
- (short)shortValue {
  return 0;
}
- (unsigned short)unsignedShortValue {
  return 0;
}
- (int)intValue {
  return 0;
}
- (unsigned int)unsignedIntValue {
  return 0;
}
- (float)floatValue {
  return 0.0;
}
- (double)doubleValue {
  return 0.0;
}

// NSCoding protocol

- (Class)classForCoder {
  return [NGNil class];
}

- (id)awakeAfterUsingCoder:(NSCoder *)_decoder {
  NGNil *globalNil = [NGNil nilObject];
  if (self == globalNil) return self;
  RELEASE(self);
  return globalNil;
}

- (void)encodeWithCoder:(NSCoder *)_encoder {
  [_encoder encodeObject:nil];
}

- (id)initWithCoder:(NSCoder *)_decoder {
  id nilValue = [_decoder decodeObject];
  NSAssert(nilValue == nil, @"error in archive ..");
  return [self init];
}

// NSCopying protocol

- (id)copyWithZone:(NSZone *)_zone {
  return RETAIN(self);
}

@end

@implementation NSObject(NGNilSupport)

+ (BOOL)isNil {
  return NO;
}
- (BOOL)isNil {
  return NO;
}

@end

void __link_NGExtensions_NGNil() {
  __link_NGExtensions_NGNil();
}
