/* 
   NGPropertyListParser.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGPropertyListParser.h,v 1.3 1998/10/02 10:15:07 helge Exp $

#import <Foundation/NSObject.h>

@class NSString, NSArray, NSDictionary, NSData;

/*
  The property list format is:

    Strings: char's without specials:  'hello', but not 'hel  lo'
             or quoted string:         '"hello world !"

    Arrays:  '(' ')'
             or '(' element ( ',' element )* ')'

    Dicts:   '{' ( dictEntry )* '}'
             dictEntry = property '=' property ';' ;

    Data:    '<' data '>', eg: '< AABB 88CC 77a7 11 >'
 */

NSString     *NGParseStringFromBuffer(const char *_buffer, unsigned _len);
NSArray      *NGParseArrayFromBuffer(const char *_buffer, unsigned _len);
NSDictionary *NGParseDictionaryFromBuffer(const char *_buffer, unsigned _len);

NSString     *NGParseStringFromData(NSData *_data);
NSArray      *NGParseArrayFromData(NSData *_data);
NSDictionary *NGParseDictionaryFromData(NSData *_data);
NSString     *NGParseStringFromString(NSString *_str);
NSArray      *NGParseArrayFromString(NSString *_str);
NSDictionary *NGParseDictionaryFromString(NSString *_str);

id NGParsePropertyListFromBuffer(const char *_buffer, unsigned _len);
id NGParsePropertyListFromData(NSData *_data);
id NGParsePropertyListFromString(NSString *_string);
id NGParsePropertyListFromFile(NSString *_path);

NSDictionary *NGParseStringsFromBuffer(const char *_buffer, unsigned _len);
NSDictionary *NGParseStringsFromData(NSData *_data);
NSDictionary *NGParseStringsFromString(NSString *_string);
NSDictionary *NGParseStringsFromFile(NSString *_path);

#import <Foundation/NSArray.h>
#import <Foundation/NSDictionary.h>

@interface NSArray(NGPropertyListParser)

+ (id)arrayWithContentsOfFile:(NSString *)_path;
- (id)initWithContentsOfFile:(NSString *)_path;

@end

@interface NSDictionary(NGPropertyListParser)

+ (id)dictionaryWithContentsOfFile:(NSString *)_path;
- (id)initWithContentsOfFile:(NSString *)_path;

@end
