/* 
   NGProxy.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGProxy.m,v 1.13 1998/12/30 08:56:06 helge Exp $

#import "common.h"
#import "NGProxy.h"

#define LOCAL_PROXY_DESCRIPTION

#if OPENSTEP
@implementation NGProxy
@end
#else
@implementation NGProxy


static inline void _subclassResponsibility(SEL _cmd) {
  NSException *exc;
  exc = [[NSException alloc]
                      initWithFormat:@"subclass should override %s",
                        sel_get_name(_cmd)];
  [exc raise];
}

static inline NSInvocation *_invocationForSelector(NGProxy *self, SEL _selector) {
  register NSInvocation *i;

  i = [NSInvocation invocationWithMethodSignature:
                      [self methodSignatureForSelector:_selector]];
  [i setTarget:self];
  [i setSelector:_selector];
  return i;
}

static inline void _invoke0(NGProxy *self, SEL _selector, void *result) {
  NSInvocation *i = _invocationForSelector(self, _selector);
  [i invoke];
  [i getReturnValue:result];
}
static inline void _invoke1(NGProxy *self, SEL _selector, void *arg1, void *result) {
  NSInvocation *i = _invocationForSelector(self, _selector);
  [i setArgument:arg1 atIndex:2];
  [i invoke];
  [i getReturnValue:result];
}
static inline void _invoke2(NGProxy *self, SEL _selector,
                            void *arg1, void *arg2, void *result) {
  NSInvocation *i = _invocationForSelector(self, _selector);
  [i setArgument:arg1 atIndex:2];
  [i setArgument:arg2 atIndex:3];
  [i invoke];
  [i getReturnValue:result];
}

// allocation

+ (id)alloc {
  return [self allocWithZone:NULL];
}

+ (id)allocWithZone:(NSZone *)_zone {
  return NSAllocateObject(self, 0, _zone);
}

- (void)dealloc {
#if !LIB_FOUNDATION_BOEHM_GC
  NSDeallocateObject((NSObject *)self);
#endif
}

- (id)autorelease {
#if !LIB_FOUNDATION_BOEHM_GC
  [NSAutoreleasePool addObject:self];
#endif
  return self;
}
+ (id)autorelease {
  return self;
}

- (oneway void)release {
#if !LIB_FOUNDATION_BOEHM_GC
#  ifdef NeXT
    if (NSExtraRefCount(self) == 1)
      [self dealloc];
    else
      NSDecrementExtraRefCountWasZero(self);
#  else
  if (NSGetExtraRefCount(self) == 1)
    [self dealloc];
  else
    NSDecrementExtraRefCountWasZero(self);
#  endif
#endif  
}
+ (oneway void)release {
}

- (id)retain {
  NSIncrementExtraRefCount(self);
  return self;
}
+ (id)retain {
  return self;
}

- (unsigned int)retainCount {
#if NeXT
  return NSExtraRefCount(self);
#else
  return NSGetExtraRefCount(self);
#endif
}
+ (unsigned int)retainCount {
  return (unsigned)-1;
}

- (NSZone *)zone {
#if NeXT
  return NSZoneFromPointer(self);
#else
  return NSZoneFromObject((NSObject *)self);
#endif
}

// reflection & forwarding

- (BOOL)isProxy {
  return YES;
}

- (void)forwardInvocation:(NSInvocation *)_invocation {
  [NSException raise:NSInvalidArgumentException
               format:@"send forwardInvocation: to NGProxy .."];
}
- (NSMethodSignature *)methodSignatureForSelector:(SEL)_selector {
  [NSException raise:NSInvalidArgumentException
               format:@"send methodSignatureForSelector: to NGProxy .."];
  return nil;
}
+ (NSMethodSignature*)methodSignatureForSelector:(SEL)_selector {
  if (!_selector)
    return nil;
  else {
    struct objc_method *mth = class_get_class_method(self, _selector);

    return mth
      ? [NSMethodSignature signatureWithObjCTypes:mth->method_types]
      : nil;
  }
}

// description

+ (NSString *)description {
  // Don't use -[NSString stringWithFormat:] method because it can cause
  // infinite recursion.
  char buffer[512];

  sprintf (buffer, "<class %s>", (char*)object_get_class_name(self));
  return [NSString stringWithCString:buffer];
}

#ifdef LOCAL_PROXY_DESCRIPTION
- (NSString *)description {
  return [NSString stringWithFormat:@"<%@[0x%08X]>",
                     NSStringFromClass(self->isa), (unsigned)self];
  /*
  return [NSString stringWithFormat:@"<%s[0x%08X]: remote=%s>",
                     (char *)class_get_class_name(self->isa), (unsigned)self,
                     [result cString]];
  char buffer[512];

  snprintf(buffer, 512, "<%s[0x%08X]>",
           (char *)class_get_class_name(self->isa), self);
  return [NSString stringWithCString:buffer];
  */
}
#endif

// class methods

+ (void)setVersion:(int)_version {
  class_set_version(self, _version);
}
+ (int)version {
  return class_get_version(self);
}

+ (Class)class {
  return self;
}
+ (Class)superclass {
  return class_get_super_class(self);
}

+ (void)poseAsClass:(Class)_class {
  class_pose_as(self, _class);
}

// NSObject protocol (abstract methods)

- (Class)class {
  Class result;
  _invoke0(self, _cmd, &result);
  return result;
}
- (Class)superclass {
  Class result;
  _invoke0(self, _cmd, &result);
  return result;
}

- (BOOL)isKindOfClass:(Class)_class {
  BOOL result;
  _invoke1(self, _cmd, &_class, &result);
  return result;
}
- (BOOL)isMemberOfClass:(Class)_class {
  BOOL result;
  _invoke1(self, _cmd, &_class, &result);
  return result;
}

- (BOOL)conformsToProtocol:(Protocol *)_protocol {
  BOOL result;
  _invoke1(self, _cmd, &_protocol, &result);
  return result;
}

- (BOOL)respondsToSelector:(SEL)_selector {
  IMP msg = _selector ? objc_msg_lookup(self, _selector) : NULL;

  if (msg != NULL) {
    return YES;
  }
  else {
    BOOL result;
    
    _invoke1(self, _cmd, &_selector, &result);
  
    return result;
  }
}

- (unsigned)hash {
  unsigned result;
  _invoke0(self, _cmd, &result);
  return result;
}
- (BOOL)isEqual:(id)_anObject {
  BOOL result;
  _invoke1(self, _cmd, &_anObject, &result);
  return result;
}

- (id)self {
  return self;
}

- (id)performSelector:(SEL)_selector {
  IMP msg = _selector ? objc_msg_lookup(self, _selector) : NULL;
  if (msg)
    return msg(self, _selector);
  else {
    id  result;
    _invoke0(self, _selector, &result);
    return result;
  }
}
- (id)performSelector:(SEL)_selector withObject:(id)_object {
  IMP msg = _selector ? objc_msg_lookup(self, _selector) : NULL;
  if (msg)
    return msg(self, _selector, _object);
  else {
    id result;
    _invoke1(self, _selector, &_object, &result);
    return result;
  }
}
- (id)performSelector:(SEL)_selector withObject:(id)_obj1 withObject:(id)_obj2 {
  IMP msg = _selector ? objc_msg_lookup(self, _selector) : NULL;
  if (msg)
    return msg(self, _selector, _obj1, _obj2);
  else {
    id result;
    _invoke2(self, _selector, &_obj1, &_obj2, &result);
    return result;
  }
}

// obsolete

- (id)perform:(SEL)_selector {
  return [self performSelector:_selector];
}
- (id)perform:(SEL)_selector withObject:(id)_object {
  return [self performSelector:_selector withObject:_object];
}
- (id)perform:(SEL)_selector withObject:(id)_obj1 withObject:(id)_obj2 {
  return [self performSelector:_selector withObject:_obj1 withObject:_obj2];
}

// forwarding

- (retval_t)forward:(SEL)_selector:(arglist_t)argFrame {
  void         *result;
  NSInvocation *invocation;
#if defined(NeXT_RUNTIME) && !defined(BROKEN_BUILTIN_APPLY) && defined(i386)
  const char   *retType;
#endif

#if NeXT_RUNTIME
  /*  On NeXT the argFrame represents the stack zone with all the arguments.
      We create a frame like that required by __builtin_apply. This is done
      by __builtin_apply_args. This builtin function also sets correctly the
      structure value return address if any. */
  arglist_t frame = __builtin_apply_args();

  frame->arg_ptr = (void*)argFrame;
  argFrame = frame;
#endif

  invocation = [NSInvocation invocationWithMethodSignature:
                               [self methodSignatureForSelector:_selector]];
  [invocation setArgumentFrame:argFrame];
  [invocation setTarget:self];
  [invocation setSelector:_selector];

  [self forwardInvocation:invocation];

  result = [invocation returnFrame];
  
#if GNU_RUNTIME
  return result;
#else // NeXT_RUNTIME
# if !defined(BROKEN_BUILTIN_APPLY) && defined(i386)
  /* Special hack to avoid pushing the poped float value back to the fp
     stack on i386 machines. This happens with NeXT runtime and 2.7.2
     compiler. If the result value is floating point don't call
     __builtin_return anymore. */
  retType = [[invocation methodSignature] methodReturnType];
  if(*retType == _C_FLT || *retType == _C_DBL) {
    long double value = *(long double*)(((char*)result) + 8);
    asm("fld %0" : : "f" (value));
  }
  else
# endif
    __builtin_return(result);
#endif /* NeXT_RUNTIME */
}

@end
#endif
