/* 
   NGTool.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGTool.h,v 1.7 1999/01/29 12:31:04 helge Exp $

#import <Foundation/NSObject.h>

@class NSArray, NSString, NSMutableArray, NSMutableDictionary;
@class NGTool;

extern NSString *NGProcessWillExitNotificationName;
extern NSString *NGProcessWillAbortNotificationName;

// option record format
typedef struct {
  NSString *name;
  NSString *longName;
  SEL      processor;         // tool method (see registerOption:value:description:)
  int      numberOfArguments;
  char     *argumentTypes;    // b-boolean, i-int, s-string
  NSString *helpText;
} NGOptionDescription;

// option types
#define NGBoolOption   'b'
#define NGIntOption    'i'
#define NGStringOption 's'

// option processing methods
#define NGRegisterOption @selector(registerOption:value:description:)
#define NGHelpOption     @selector(showHelpOption:value:description:)
#define NGTopicLogOption @selector(logTopicOption:value:description:)

// option record entries

#define NGFileOption \
  { @"*", nil, @selector(registerFile:value:description:), 0, NULL, @"a file" }

#define NGLastOption { nil, nil, NULL, 0, NULL, nil }

// tool main function
typedef void (*NGToolFunction)(NGTool *tool, NSArray *args, int argc, char **argv);

@interface NGTool : NSObject
{
  NGToolFunction func;

  NSProcessInfo *processInfo;
  NSString      *executablePath;

  char                **argv;
  NSMutableDictionary *options;
  NSMutableArray      *files;
}

+ (NGTool *)tool;

- (id)initWithArguments:(char **)_argv count:(int)_argc environment:(char **)_env;

// accessors

- (void)setToolFunction:(NGToolFunction)_func;

// running

- (void)run;

// exiting & aborting

- (void)exitWithCode:(int)_exitCode;
- (void)coreDump;
- (void)prepareForExit;
- (void)prepareForAbort;

// tool info

- (NSString *)toolName;
- (NSString *)executablePath;

// argument processing

- (void)parseOptions:(NGOptionDescription *)_optionDescriptions;

- (void)registerOption:(NSString *)_option value:(NSArray *)_args
  description:(NGOptionDescription *)_description;

- (void)registerFile:(NSString *)_fileName value:(NSArray *)_alwaysNil
  description:(NGOptionDescription *)_description;

- (void)showHelpOption:(NSString *)_option value:(NSArray *)_value
  description:(NGOptionDescription *)_description;

- (BOOL)isOptionSet:(NSString *)_optionName;
- (id)valueOfOption:(NSString *)_optionName;
- (BOOL)isFlagOptionSet:(NSString *)_optionName;
- (int)intValueOfOption:(NSString *)_optionName;

- (NSArray *)fileArguments;

@end
