/* 
   NSAttributedString.h

   Copyright (C) 1998 MDlink online service center
   All rights reserved.

   Author: Jan Reichmann <jan@mdlink.de>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NSAttributedString.h,v 1.5 1998/10/15 17:15:26 helge Exp $

#if OPENSTEP
#import <Foundation/NSAttributedString.h>
#else
#import <Foundation/NSObject.h>
#import <Foundation/NSException.h>
#import <Foundation/NSZone.h>

@class NSString, NSDictionary, NSCoder, NSException;

@interface NSAttributedString : NSObject <NSCopying, NSMutableCopying, NSCoding>
{
}

// Initializing an NSAttributedString

- (id)initWithString:(NSString *)_string;
- (id)initWithString:(NSString *)_string attributes:(NSDictionary *)_attributes;
- (id)initWithAttributedString:(NSAttributedString *)_attributedString;

// Retrieving character information

- (NSString *)string;
- (unsigned int)length;

// Retrieving attribute information

- (id)attribute:(NSString *)_attributeName
      atIndex:(unsigned int)_index
      effectiveRange:(NSRange *)_range;

- (id)attribute:(NSString *)_attributeName
      atIndex:(unsigned int)_index
      longestEffectiveRange:(NSRange *)_range
      inRange:(NSRange)_rangeLimit;

- (NSDictionary *)attributesAtIndex:(unsigned int)_index
                  effectiveRange:(NSRange *)_range;

- (NSDictionary *)attributesAtIndex:(unsigned int)_index
                  longestEffectiveRange:(NSRange *)_range
                  inRange:(NSRange)_rangeLimit;

// Comparing attributed strings

- (BOOL)isEqual:(id)_anObject;
- (BOOL)isEqualToAttributedString:(NSAttributedString *)_otherString;

// Extracting a substring

- (NSAttributedString *)attributedSubstringFromRange:(NSRange)_range;

@end /* NSAttributedString */

  
@interface NSMutableAttributedString : NSAttributedString
{
}

// Retrieving character information

- (NSMutableString *)mutableString;

// Changing characters

- (void)replaceCharactersInRange:(NSRange)aRange withString:(NSString *)aString;
- (void)deleteCharactersInRange:(NSRange)aRange;

// Changing attributes

- (void)setAttributes:(NSDictionary *)attributes range:(NSRange)aRange;
- (void)addAttribute:(NSString *)name
        value:(id)value
        range:(NSRange)aRange;
- (void)addAttributes:(NSDictionary *)attributes range:(NSRange)aRange;
- (void)removeAttribute:(NSString *)name range:(NSRange)aRange;

//Changing characters and attributes

- (void)appendAttributedString:(NSAttributedString *)_attributedString;
- (void)insertAttributedString:(NSAttributedString *)_attributedString
        atIndex:(unsigned int)index;
- (void)replaceCharactersInRange:(NSRange)aRange
        withAttributedString:(NSAttributedString *)_attributedString;
- (void)setAttributedString:(NSAttributedString *)_attributedString;

// Grouping changes

- (void)beginEditing;
- (void)endEditing;

@end /* NSMutableAttributedString */

@interface NSAttributedStringException : NSException
@end /* NSAttributedStringException */
#endif

/*
  Local Variables:
  c-basic-offset: 4
  tab-width: 8
  End:
*/
