/* 
   NSAttributedString.m

   Copyright (C) 1998 MDlink online service center
   All rights reserved.

   Author: Jan Reichmann <jan@mdlink.de>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NSAttributedString.m,v 1.5 1998/10/02 16:51:08 helge Exp $

#import "common.h"
#import "NSAttributedString.h"
#import "NSConcreteAttributedString.h"

/*
  NSAttributedString class cluster

  Currently there is only one subclass pair implemented called
  NSConcreteShortAttributedString/_NSConcreteShortMutableAttributedString.
  This classes handle attributed strings for only 256 combinations of
  attributes.
  The attributes (NSDictionarys) are stored in an id - array, so this
  implementation allow a really fast access to the attributes.
  For more information see _NSConcreteShortAttributedString.
  If you need more combinations, you have to complete _NSLongAttributedString.
  This implementation stores the attributes in Ranges.

  The core-functions from the _NSConcreteShortAttributedString/
  _NSConcreteShortMutableAttributedString and some some functions from
  the _NSLongAttributedString are defined in NSShortAttributedStringFunctions
  and NSLongAttributedStringFunctions. I have done this because the
  _NSConcreteShortMutableAttributedString doesn't inherit from
  _NSConcreteShortAttributedString and so I could avoid to write the same 
  Methods sveral times.
*/

@implementation NSAttributedString

+ (id)allocWithZone:(NSZone *)_zone {
  return NSAllocateObject((self == [NSAttributedString class]) ?
                           [_NSConcreteShortAttributedString class] : self,
                           0, _zone);
}

- (id)initWithString:(NSString *)_string {
  return [self initWithString:_string attributes:[NSDictionary dictionary]];
}

- (id)initWithString:(NSString *)_string attributes:(NSDictionary *)_attributes {
  return [self subclassResponsibility:_cmd];
}

- (id)initWithAttributedString:(NSAttributedString *)_attributedString {
  return [self subclassResponsibility:_cmd];
}

// accessors

- (NSString *)string {
  return [self subclassResponsibility:_cmd];
}

- (unsigned int)length {
  [self subclassResponsibility:_cmd];
  return 0;
}

// Retrieving attribute information

- (id)attribute:(NSString *)_attributeName
      atIndex:(unsigned int)_index
      effectiveRange:(NSRange *)_range {
  
  return [self subclassResponsibility:_cmd];
}

- (id)attribute:(NSString *)_attributeName
      atIndex:(unsigned int)_index
      longestEffectiveRange:(NSRange *)_range
      inRange:(NSRange)_rangeLimit {

  return [self subclassResponsibility:_cmd];
}

- (NSAttributedString *)attributedSubstringFromRange:(NSRange)_range {
  return [self subclassResponsibility:_cmd];
}
 
- (NSDictionary *)attributesAtIndex:(unsigned int)_index
                  effectiveRange:(NSRange *)_range {
  return [self subclassResponsibility:_cmd];
}
 
- (NSDictionary *)attributesAtIndex:(unsigned int)_index
                  longestEffectiveRange:(NSRange *)_range
                  inRange:(NSRange)_rangeLimit {
  return [self subclassResponsibility:_cmd];
}

- (BOOL)isEqual:(id)_anObject {
  return ([_anObject isKindOfClass:[NSAttributedString class]])
    ? [self isEqualToAttributedString:_anObject]
    : NO;
}

- (BOOL)isEqualToAttributedString:(NSAttributedString *)_otherString {
  [self subclassResponsibility:_cmd];
  return NO;
}

- (void)encodeWithCoder:(NSCoder *)_encoder {
  [self subclassResponsibility:_cmd];
}
- (id)initWithCoder:(NSCoder *)_decoder {
  return [self subclassResponsibility:_cmd];
}

- (NSString *)description {
  return [self subclassResponsibility:_cmd];
}

//  NSCopying, NSMutableCopying

- (id)copyWithZone:(NSZone *)_zone {
  return [self subclassResponsibility:_cmd];
}
- (id)mutableCopyWithZone:(NSZone *)_zone {
  return [self subclassResponsibility:_cmd];
}

@end /* NSAttributedString */

@implementation NSMutableAttributedString

+ (id)allocWithZone:(NSZone *)zone {
  return NSAllocateObject((self == [NSMutableAttributedString class]) ? 
                          [_NSConcreteShortMutableAttributedString class] : self,
                          0, zone);
}

// accessors

- (NSMutableString *)mutableString {
  return [self subclassResponsibility:_cmd];
}

// Changing characters
- (void)replaceCharactersInRange:(NSRange)aRange withString:(NSString *)aString {
  [self subclassResponsibility:_cmd];
}
- (void)deleteCharactersInRange:(NSRange)aRange {
  [self subclassResponsibility:_cmd];
}

//Changing attributes
- (void)setAttributes:(NSDictionary *)attributes range:(NSRange)aRange {
  [self subclassResponsibility:_cmd];
}

- (void)addAttribute:(NSString *)name
        value:(id)value
        range:(NSRange)aRange {
  
  [self subclassResponsibility:_cmd];
}

- (void)addAttributes:(NSDictionary *)attributes range:(NSRange)aRange {
  [self subclassResponsibility:_cmd];
}

- (void)removeAttribute:(NSString *)name range:(NSRange)aRange {
  [self subclassResponsibility:_cmd];
}

// Changing characters and attributes

- (void)appendAttributedString:(NSAttributedString *)attributedString {
  [self subclassResponsibility:_cmd];
}
- (void)insertAttributedString:(NSAttributedString *)attributedString
        atIndex:(unsigned int)index {
  [self subclassResponsibility:_cmd];
}
- (void)replaceCharactersInRange:(NSRange)aRange
        withAttributedString:(NSAttributedString *)attributedString {
  [self subclassResponsibility:_cmd];
}
- (void)setAttributedString:(NSAttributedString *)attributedString {
  [self subclassResponsibility:_cmd];
}

// Grouping changes

- (void)beginEditing {
}
- (void)endEditing {
}

@end /* NSMutableAttributedString */

@implementation NSAttributedStringException : NSException
@end /* NSAttributedStringException */

/*
  Local Variables:
  c-basic-offset: 4
  tab-width: 8
  End:
*/
