/* 
   NSCalendarDate+misc.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NSCalendarDate+misc.m,v 1.8 1999/03/16 11:28:58 helge Exp $

#import "common.h"
#import "NSCalendarDate+misc.h"

#define NUMBER_OF_SECONDS_IN_DAY (24 * 60 * 60)

@implementation NSCalendarDate(misc)

- (int)weekOfYear {
  int            whereToStart[] = { 6, 7, 8, 9, 10, 4, 5 };
  NSCalendarDate *janFirst      = nil;
  int            year           = [self yearOfCommonEra];
  int            day            = [self dayOfYear]; 
  int            weekOfYear;
  
  janFirst  = [NSCalendarDate dateWithYear:year month:1 day:1 
                              hour:0 minute:0 second:0
                              timeZone:[self timeZone]];
  
  weekOfYear = (day + whereToStart[[janFirst dayOfWeek]]) / 7;

  if (weekOfYear == 0) {
    NSCalendarDate *silvesterLastYear;

    silvesterLastYear = [NSCalendarDate dateWithYear:(year - 1)
                                        month:12 day:31 
                                        hour:0 minute:0 second:0
                                        timeZone:[self timeZone]];
    return [silvesterLastYear weekOfYear];
  }

#if 0  
  if (weekOfYear == 53) {
    NSCalendarDate *nextJanFirst = nil;
    int            janYear       = year + 1;
    int            janDay;
    int            week;

    nextJanFirst = [NSCalendarDate dateWithYear:janYear
                                   month:1 day:1
                                   hour:0 minute:0 second:0
                                   timeZone:[self timeZone]];

    janDay = [nextJanFirst dayOfYear];
    week   = (janDay + whereToStart[[nextJanFirst dayOfWeek]]) / 7;

    if (week == 1)
      return 52;
  }
#endif  
  return weekOfYear;
}

- (NSCalendarDate *)mondayOfWeek {
  NSTimeInterval tv;
  NSCalendarDate *monday;
  int            dayOfWeek = [self dayOfWeek];
        
  if (dayOfWeek == 0) dayOfWeek = 7; // readjust Sunday

  tv = (1 - dayOfWeek) * NUMBER_OF_SECONDS_IN_DAY;
  tv = [self timeIntervalSince1970] + tv;
  monday = [NSCalendarDate dateWithTimeIntervalSince1970:tv];
  [monday setTimeZone:[self timeZone]];
  return monday;
}

- (NSCalendarDate *)beginOfDay {
  return [NSCalendarDate dateWithYear:[self yearOfCommonEra]
                         month:       [self monthOfYear]
                         day:         [self dayOfMonth]
                         hour:0 minute:0 second:0
                         timeZone:    [self timeZone]];
}

- (NSCalendarDate *)endOfDay {
  return [NSCalendarDate dateWithYear:[self yearOfCommonEra]
                         month:       [self monthOfYear]
                         day:         [self dayOfMonth]
                         hour:23 minute:59 second:59
                         timeZone:    [self timeZone]];
}

- (BOOL)isDateOnSameDay:(NSCalendarDate *)_date {
  if ([self dayOfYear]       != [_date dayOfYear])       return NO;
  if ([self yearOfCommonEra] != [_date yearOfCommonEra]) return NO;
  return YES;
}
- (BOOL)isDateInSameWeek:(NSCalendarDate *)_date {
  if ([self weekOfYear]      != [_date weekOfYear])      return NO;
  if ([self yearOfCommonEra] != [_date yearOfCommonEra]) return NO;
  return YES;
}

- (NSCalendarDate *)yesterday {
  return [self dateByAddingYears:0 months:0 days:-1 hours:0 minutes:0 seconds:0];
}
- (NSCalendarDate *)tomorrow {
  return [self dateByAddingYears:0 months:0 days:1 hours:0 minutes:0 seconds:0];
}

- (NSCalendarDate *)lastYear {
  return [self dateByAddingYears:-1 months:0 days:0 hours:0 minutes:0 seconds:0];
}
- (NSCalendarDate *)nextYear {
  return [self dateByAddingYears:1 months:0 days:0 hours:0 minutes:0 seconds:0];
}

- (NSCalendarDate *)hour:(int)_hour minute:(int)_minute second:(int)_second {
  return [NSCalendarDate dateWithYear:[self yearOfCommonEra]
                         month:       [self monthOfYear]
                         day:         [self dayOfMonth]
                         hour:_hour minute:_minute second:_second
                         timeZone:    [self timeZone]];
}
- (NSCalendarDate *)hour:(int)_hour minute:(int)_minute {
  return [self hour:_hour minute:_minute second:0];
}

@end

// linking

void __link_NSCalendarDate_misc(void) {
  __link_NSCalendarDate_misc();
}
