/* 
   NSSet+enumerator.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Jan Reichmann (jan@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NSSet+enumerator.m,v 1.5 1999/01/08 13:43:40 helge Exp $


#import "common.h"
#import "NSSet+enumerator.h"
#import "NGNil.h"

@implementation NSSet(enumerator)

- (id)initWithObjectsFromEnumerator:(NSEnumerator *)_enumerator {
  NSMutableSet *set = nil;

  set = [[NSMutableSet alloc] initWithObjectsFromEnumerator:_enumerator];
  self = [self initWithSet:set];
  RELEASE(set); set = nil;
  return self;
}

// mapping

- (NSArray *)mappedArrayUsingSelector:(SEL)_selector {
  NSArray *array = nil;
  NSSet   *set   = nil;

  set = [self mappedSetUsingSelector:_selector];
  if (set) {
    array = [[NSArray allocWithZone:[self zone]]
                      initWithObjectsFromEnumerator:[set objectEnumerator]];
  }
  set = nil;
  return AUTORELEASE(array);
}
- (NSArray *)mappedArrayUsingSelector:(SEL)_selector withObject:(id)_object {
  NSArray *array = nil;
  NSSet   *set   = nil;

  set = [self mappedSetUsingSelector:_selector withObject:_object];
  if (set) {
    array = [[NSArray allocWithZone:[self zone]]
                      initWithObjectsFromEnumerator:[set objectEnumerator]];
  }
  set = nil;
  return AUTORELEASE(array);
}

- (NSSet *)mappedSetUsingSelector:(SEL)_selector {
  NSMutableSet *set   = [NSMutableSet setWithCapacity:[self count]];
  NSEnumerator *e     = [self objectEnumerator];
  id           object = nil;

  while ((object = [e nextObject])) {
    object = [object performSelector:_selector];

    [set addObject:object ? object : [NGNil nilObject]];
  }
  return set;
}
- (NSSet *)mappedSetUsingSelector:(SEL)_selector withObject:(id)_object {
  NSMutableSet *set = [NSMutableSet setWithCapacity:[self count]];
  NSEnumerator *e     = [self objectEnumerator];
  id           object = nil;

  while ((object = [e nextObject])) {
    object = [object performSelector:_selector withObject:_object];

    [set addObject:object ? object : [NGNil nilObject]];
  }
  return set;
}

@end

@implementation NSMutableSet(enumerator) 

- (id)initWithObjectsFromEnumerator:(NSEnumerator *)_enumerator {
  if ((self = [self init])) {
    id obj = nil;
     
    while ((obj = [_enumerator nextObject]))
      [self addObject:obj];
  }
  return self;
}

@end

void __link_NGExtensions_NSSetEnumerator() {
  __link_NGExtensions_NSSetEnumerator();
}
