/* 
   NSShortAttributedStringFunctions.h

   Copyright (C) 1998 MDlink online service center
   All rights reserved.

   Author: Jan Reichmann <jan@mdlink.de>

   This file is part of libFoundation.

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
//$Id: NSShortAttributedStringFunctions.h,v 1.4 1998/10/02 10:15:14 helge Exp $

#import "Foundation/Foundation.h"
#import "NSAttributedStringTypeDefinitions.h"

// Functions

// Create ShortAttributedString

NSShortAttributedString *NSCreateShortAStringFromString(
                                                        NSString *_string,
                                                        NSDictionary *_attr,
                                                        void *_zone);

NSShortAttributedString *NSCreateShortAStringWithShortAString(
                                                              NSShortAttributedString *_string,
                                                              void *_zone);

NSShortAttributedString *NSCreateShortAStringWithLongAString(
                                                             NSLongAttributedString *_longString);

// Free 
void NSFreeShortAttributedString(NSShortAttributedString *_string);

// Retrieving attribute information

NSDictionary *NSShortAllAttributesAtIndex(
                                          NSShortAttributedString *_string,
                                          unsigned int            _index,
                                          NSRange                 *_longestEffectiveRange,
                                          NSRange                 *_rangeLimit);

id  NSShortAttributeAtIndex(NSShortAttributedString *_string,
                            NSString                *_attributeName,
                            unsigned int            _index,
                            NSRange                 *_longestEffectiveRange,
                            NSRange                 *_rangeLimit);

// Comparing strings

BOOL NSCompareShortAttributedStrings(NSShortAttributedString *_string,
                                     NSShortAttributedString *_otherString);
// Substring

NSShortAttributedString *NSShortAttributedSubstring(
                                                    NSShortAttributedString *_string,
                                                    NSRange                 _range);

// Changing

void NSShortReplaceWithString(NSShortAttributedString *_string,
                              NSRange                 _range,
                              NSString                *string);

void NSShortReplaceWithAttributedString(
                                        NSShortAttributedString *_string,
                                        NSRange                 _range,
                                        NSShortAttributedString *_otherString);

void NSShortSetAttributes(NSShortAttributedString *_string,
                          NSRange                 _range,
                          NSDictionary            *_attributes);

void NSShortAddAttributes(NSShortAttributedString *_string,
                          NSRange                 _range,
                          NSDictionary            *_attributes);

void NSShortRemoveAttribute(NSShortAttributedString *_string,
                            NSRange                 _range,
                            NSString                *_name);

/*
  Local Variables:
  c-basic-offset: 4
  tab-width: 8
  End:
*/
