/* 
   NGDatagramPacket.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGDatagramPacket.m,v 1.5 1999/01/27 17:02:06 helge Exp $

#include "common.h"
#include "NGDatagramPacket.h"

@implementation NGDatagramPacket

+ (id)packetWithData:(NSData *)_data {
  return AUTORELEASE([[self alloc] initWithData:_data]);
}
+ (id)packetWithBytes:(const void *)_bytes size:(int)_packetSize {
  return AUTORELEASE([[self alloc] initWithBytes:_bytes size:_packetSize]);
}

- (id)initWithBytes:(const void *)_bytes size:(int)_packetSize {
  return [self initWithData:[NSData dataWithBytes:_bytes length:_packetSize]];
}
- (id)initWithData:(NSData *)_data {
  if ((self = [self init])) {
    self->packet = [_data copyWithZone:[self zone]];
  }
  return self;
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->packet);   self->packet   = nil;
  RELEASE(self->sender);   self->sender   = nil;
  RELEASE(self->receiver); self->receiver = nil;
  [super dealloc];
}
#endif

// accessors

- (void)setSender:(id<NGSocketAddress>)_address {
  ASSIGN(self->sender, _address);
}
- (id<NGSocketAddress>)sender {
  return self->sender;
}

- (void)setReceiver:(id<NGSocketAddress>)_address {
  ASSIGN(self->receiver, _address);
}
- (id<NGSocketAddress>)receiver {
  return self->receiver;
}

- (void)setData:(NSData *)_data {
  ASSIGN(self->packet, _data);
}
- (NSData *)data {
  return self->packet;
}

- (int)packetSize {
  return [self->packet length];
}

// operations

- (void)reverseAddresses {
  id oldSender = RETAIN([self sender]);
  [self setSender:[self receiver]];
  [self setReceiver:oldSender];
  RELEASE(oldSender); oldSender = nil;
}

// description

- (NSString *)description {
  return [NSString stringWithFormat:@"<%@[0x%08X]: from=%@ to=%@ size=%i>",
                     NSStringFromClass([self class]), (unsigned)self,
                     [self sender], [self receiver], [self packetSize]];
}

@end
