/* 
   NGInternetSocketDomain.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGInternetSocketDomain.m,v 1.7 1999/01/27 17:02:07 helge Exp $

#include "common.h"
#include "NGInternetSocketDomain.h"
#include "NGInternetSocketAddress.h"

@implementation NGInternetSocketDomain

static NGInternetSocketDomain *domain = nil;

+ (void)initialize {
  BOOL isInitialized = NO;
  if (!isInitialized) {
    isInitialized = YES;
    domain = [[NGInternetSocketDomain alloc] init];
  }
}
  
+ (id)domain {
  return domain;
}

// NGSocketDomain

- (id<NGSocketAddress>)addressWithRepresentation:(void *)_data
  size:(unsigned int)_size {

  NGInternetSocketAddress *address = nil;

  if ([self addressRepresentationSize] != _size) {
    NSLog(@"%@: invalid address size %i ..", NSStringFromSelector(_cmd), _size);
    return nil;
  }

  address = [[NGInternetSocketAddress allocWithZone:[self zone]]
                                      initWithDomain:self
                                      internalRepresentation:_data
                                      size:_size];
  return AUTORELEASE(address);
}

- (void)prepareAddress:(id<NGSocketAddress>)_address
  forBindWithSocket:(id<NGSocket>)_socket {

  // nothing to prepare
}
- (void)cleanupAddress:(id<NGSocketAddress>)_address
  afterCloseOfSocket:(id<NGSocket>)_socket {

  // nothing to cleanup
}

- (int)socketDomain {
  return AF_INET;
}

- (int)addressRepresentationSize {
  return sizeof(struct sockaddr_in);
}

- (int)protocol {
  return 0;
}

// NSCopying

- (id)copyWithZone:(NSZone *)_zone {
  return RETAIN(self);
}

// NSCoding

- (void)encodeWithCoder:(NSCoder *)_encoder {
}
- (id)initWithCoder:(NSCoder *)_decoder {
  RELEASE(self); self = nil;
  return RETAIN(domain);
}

- (id)awakeAfterUsingCoder:(NSCoder *)_decoder {
  if (self != domain) {
    RELEASE(self); self = nil;
    return RETAIN(domain);
  }
  else
    return self;
}

// description

- (NSString *)description {
  return [NSString stringWithFormat:@"<InternetDomain[0x%08X]>", (unsigned)self];
}

@end
