/* 
   NGLocalSocketAddress.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGLocalSocketAddress.m,v 1.9 1999/01/29 14:27:25 helge Exp $

#if !defined(WIN32)

#include "common.h"
#include "NGSocketExceptions.h"
#include "NGLocalSocketAddress.h"
#include "NGLocalSocketDomain.h"

#ifndef SUN_LEN
#define SUN_LEN(su) \
        (sizeof(*(su)) - sizeof((su)->sun_path) + strlen((su)->sun_path))
#endif

#ifndef AF_LOCAL
#define AF_LOCAL AF_UNIX
#endif

static NSString *socketDirectoryPath = @"/tmp";

@implementation NGLocalSocketAddress

+ (id)addressWithPath:(NSString *)_path {
  return AUTORELEASE([[self alloc] initWithPath:_path]);
}
+ (id)address {
  return AUTORELEASE([[self alloc] init]);
}

- (id)initWithPath:(NSString *)_path {
  if ((self = [super init])) {
    address.sun_family = [[self domain] socketDomain];
    [_path getCString:address.sun_path maxLength:(sizeof(address.sun_path) - 1)];
  }
  return self;
}

- (id)init {
  int      addressCounter = 0;
  NSString *newPath;
  
  newPath = [NSString stringWithFormat:@"_ngsocket_%08X_%08X_%03d",
                        getpid(), [NSThread currentThread], addressCounter];
  newPath = [socketDirectoryPath stringByAppendingPathComponent:newPath];

  return [self initWithPath:newPath];
}

- (id)initWithDomain:(id)_domain
  internalRepresentation:(void *)_representation
  size:(int)_length {
  // this method is used by the address factory

  struct sockaddr_un *nun = _representation;

  return [self initWithPath:[NSString stringWithCString:nun->sun_path]];
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  [super dealloc];
}
#endif

// accessors

- (NSString *)path {
  return [NSString stringWithCString:address.sun_path];
}

// operations

- (void)deletePath {
  unlink(address.sun_path);
}

// NGSocketAddress protocol

- (void *)internalAddressRepresentation {
  return &address;
}
- (int)addressRepresentationSize { // varies in length
  return SUN_LEN(&address);
}
- (id)domain {
  return [NGLocalSocketDomain domain];
}

// testing for equality

- (BOOL)isEqualToAddress:(NGLocalSocketAddress *)_addr {
  return [[_addr path] isEqualToString:[self path]];
}

- (BOOL)isEqual:(id)_object {
  if (_object == self) return YES;
  if ([_object class] != [self class]) return NO;
  return [self isEqualToAddress:_object];
}

// NSCopying

- (id)copyWithZone:(NSZone *)_zone {
  return RETAIN(self);
}

// NSCoding

- (void)encodeWithCoder:(NSCoder *)_encoder {
  [_encoder encodeObject:[[NSHost currentHost] name]];
  [_encoder encodeObject:[self path]];
}

- (id)initWithCoder:(NSCoder *)_decoder {
  NSString *hostName = [_decoder decodeObject];
  NSString *path     = [_decoder decodeObject];

  NSAssert([path isKindOfClass:[NSString class]], @"path must be a string ..");

  if (![hostName isEqualToString:[[NSHost currentHost] name]]) {
    NSLog(@"unarchived local socket address on a different host, "
          @"encoded on %@, decoded on %@ (path=%@)",
          hostName, [[NSHost currentHost] name], path);
  }

  return [self initWithPath:path];
}

// description

- (NSString *)stringValue {
  NSString *p = [self path];
  return [p length] == 0 ? @"*" : (id)p;
}

- (NSString *)description {
  NSString *p = [self path];
  
  if ([p length] == 0)
    p = @"[unnamed]";

  return [NSString stringWithFormat:@"<0x%08X[%@]: %@>",
                     self, NSStringFromClass([self class]),
                     p];
}

@end

#endif // !WIN32
