/* 
   NGSocketExceptions.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGSocketExceptions.h,v 1.10 1999/01/27 17:02:08 helge Exp $

#ifndef __NGNet_NGSocketExceptions_H__
#define __NGNet_NGSocketExceptions_H__

#import <Foundation/NSException.h>
#include <NGStreams/NGStreamExceptions.h>
#include <NGNet/NGSocketProtocols.h>

/*
  Exceptions:

    NGIOException
      NGSocketException
        NGSocketBindException
          NGSocketAlreadyBoundException
          NGCouldNotBindSocketException
        NGSocketConnectionException
          NGSocketNotConnectedException
          NGSocketAlreadyConnectedException
          NGCouldNotConnectException
        NGSocketOptionException
          NGCouldNotSetSocketOptionException
          NGCouldNotGetSocketOptionException
        NGCouldNotResolveHostNameException
        NGDidNotFindServiceException
        NGSocketIsAlreadyListeningException
        NGCouldNotListenException
        NGCouldNotAcceptException
        NGInvalidSocketDomainException
        NGCouldNotCreateSocketException
      NGStreamException
        NGEndOfStreamException
          NGSocketShutdownException
            NGSocketConnectionResetException
            NGSocketTimedOutException
*/

@interface NGSocketException : NGIOException
{
@protected
  id<NGSocket> socket;
}

- (id)init;
- (id)initWithReason:(NSString *)_reason;
- (id)initWithReason:(NSString *)_reason socket:(id<NGSocket>)_socket;

- (id<NGSocket>)socket;

@end

@interface NGCouldNotResolveHostNameException : NGSocketException
{
@protected
  NSString *hostName;
}

- (id)initWithHostName:(NSString *)_name reason:(NSString *)_reason;

- (NSString *)hostName;

@end

@interface NGDidNotFindServiceException : NGSocketException
{
@protected
  NSString *serviceName;
}

- (id)init;
- (id)initWithServiceName:(NSString *)_service;

- (NSString *)serviceName;

@end

@interface NGInvalidSocketDomainException : NGSocketException
{
@protected
  id<NGSocketDomain> domain;
}

- (id)initWithReason:(NSString *)_reason
  socket:(id<NGSocket>)_socket domain:(id<NGSocketDomain>)_domain;

@end

@interface NGCouldNotCreateSocketException : NGSocketException
{
@protected
  id<NGSocketDomain> domain;
}

- (id)init;
- (id)initWithReason:(NSString *)_reason domain:(id<NGSocketDomain>)_domain;

@end

// ******************** bind ***********************

@interface NGSocketBindException : NGSocketException
@end

@interface NGSocketAlreadyBoundException : NGSocketBindException
@end

@interface NGCouldNotBindSocketException : NGSocketBindException
{
@protected
  id<NGSocketAddress> address;
}

- (id)initWithReason:(NSString *)_reason
  socket:(id<NGSocket>)_socket address:(id<NGSocketAddress>)address;

- (id<NGSocketAddress>)address;

@end

// ******************** connect ********************

@interface NGSocketConnectException : NGSocketException
@end

@interface NGSocketNotConnectedException : NGSocketConnectException
@end

@interface NGSocketAlreadyConnectedException : NGSocketConnectException
@end

@interface NGCouldNotConnectException : NGSocketConnectException
{
@protected
  id<NGSocketAddress> address;
}

- (id)initWithReason:(NSString *)_reason
  socket:(id<NGActiveSocket>)_socket
  address:(id<NGSocketAddress>)address;

- (id<NGSocketAddress>)address;

@end

// ******************** listen ********************

@interface NGSocketIsAlreadyListeningException : NGSocketException
@end

@interface NGCouldNotListenException : NGSocketException
@end

// ******************** accept ********************

@interface NGCouldNotAcceptException : NGSocketException
@end

// ******************** options ********************

@interface NGSocketOptionException : NGSocketException
{
@protected
  int option;
  int level;
}

- (id)init;
- (id)initWithReason:(NSString *)_reason option:(int)_option level:(int)_level;

@end

@interface NGCouldNotSetSocketOptionException : NGSocketOptionException
@end

@interface NGCouldNotGetSocketOptionException : NGSocketOptionException
@end

// ******************** socket closed **************

@interface NGSocketShutdownException : NGEndOfStreamException

- (id)initWithReason:(NSString *)_reason;
- (id)initWithReason:(NSString *)_reason socket:(id<NGActiveSocket>)_socket;
- (id)initWithSocket:(id<NGActiveSocket>)_socket;

- (id<NGActiveSocket>)socket;

@end

@interface NGSocketTimedOutException : NGSocketShutdownException
@end

@interface NGSocketConnectionResetException : NGSocketShutdownException
@end

#endif /* __NGNet_NGSocketExceptions_H__ */
