/* 
   NGSocketProtocols.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGSocketProtocols.h,v 1.9 1999/01/27 15:47:59 helge Exp $

#ifndef __NGNet_NGSocketProtocols_H__
#define __NGNet_NGSocketProtocols_H__

#import <Foundation/NSObject.h>
#include <NGStreams/NGStreamProtocols.h>

// addresses

@protocol NGSocketAddress < NSObject >

- (void *)internalAddressRepresentation;
- (int)addressRepresentationSize;
- (id)domain; // (a NGSocketDomain)

// needed by socket address factory:
- (id)initWithDomain:(id)_domain
  internalRepresentation:(void *)_representation
  size:(int)_length;

@end

// sockets

@protocol NGSocket < NSObject >

- (id<NGSocketAddress>)localAddress;
- (void)bindToAddress:(id<NGSocketAddress>)_localAddress;
- (void)close;

@end

// domains

@protocol NGSocketDomain < NSObject >

- (id<NGSocketAddress>)addressWithRepresentation:(void *)_data
  size:(unsigned int)_size;

- (int)socketDomain;
- (int)addressRepresentationSize;
- (int)protocol;

// these two methods manage resources associated with addresses
// (primarily the files used for AF_LOCAL sockets)
- (void)prepareAddress:(id<NGSocketAddress>)_address
  forBindWithSocket:(id<NGSocket>)_socket;
- (void)cleanupAddress:(id<NGSocketAddress>)_address
  afterCloseOfSocket:(id<NGSocket>)_socket;

@end

// concrete sockets

@protocol NGActiveSocket < NGSocket, NGStream, NGByteSequenceStream >

- (void)connectToAddress:(id<NGSocketAddress>)_address;
- (void)shutdown;

- (BOOL)isConnected;

- (id<NGSocketAddress>)remoteAddress;

@end

@protocol NGPassiveSocket < NGSocket >

- (void)listenWithBacklog:(int)_backlogSize;
- (id<NGActiveSocket>)accept;

@end

// packets

@protocol NGDatagramPacket < NSObject >

- (void)setSender:(id<NGSocketAddress>)_address;
- (id<NGSocketAddress>)sender;
- (void)setReceiver:(id<NGSocketAddress>)_address;
- (id<NGSocketAddress>)receiver;

- (NSData *)data;

@end

@protocol NGDatagramPacketFactory < NSObject >

- (id<NGDatagramPacket>)packetWithData:(NSData *)_data;

- (id<NGDatagramPacket>)
  packetWithBytes:(const void *)_bytes
  size:(int)_packetSize;

@end

#endif /* __NGNet_NGSocketProtocols_H__ */
