/* 
   NGCTextStream.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGCTextStream.h,v 1.5 1999/01/27 16:11:15 helge Exp $

#ifndef __NGStreams_NGCTextStream_H__
#define __NGStreams_NGCTextStream_H__

#include <NGStreams/NGStream.h>
#include <NGStreams/NGTextStream.h>
#include <NGStreams/NGTextStreamProtocols.h>
#include <NGStreams/NGFilterStream.h>

extern id<NGExtendedTextInputStream>  NGTextIn;
extern id<NGExtendedTextOutputStream> NGTextOut;
extern id<NGExtendedTextOutputStream> NGTextErr;
extern void NGInitTextStdio(void);

/*
  NGCTextStream

  NGCTextStream is a text stream which operates in the operation systems
  default encoding (it returns the bytes read from the source as characters).
  Note that the results of the unicode-methods do not necessarily represent a
  valid unicode character. This is only the case for character codes in the
  7bit ASCII set.
  NGCTextStream never returns a character value above 255.
  
  To retrieve correctly converted unicode characters use the NGTextStream class.
*/

@interface NGCTextStream : NGTextStream
{
@private
  id<NGStream>        source; // retained
  NGIOReadMethodType  readBytes;
  NGIOWriteMethodType writeBytes;
  void                (*flushBuffer)(id, SEL);
}

+ (id)textStreamWithInputSource:(id<NGInputStream>)_source;
+ (id)textStreamWithOutputSource:(id<NGOutputStream>)_source;
+ (id)textStreamWithSource:(id<NGStream>)_stream;
- (id)initWithSource:(id<NGStream>)_stream;
- (id)initWithInputSource:(id<NGInputStream>)_source;
- (id)initWithOutputSource:(id<NGOutputStream>)_source;

// accessors

- (id<NGStream>)source;

// operations

- (void)close; // forwarded to source

// NGTextInputStream, NGExtendedTextInputStream

- (unichar)readCharacter;
- (char)readChar;

- (NSString *)readLineAsString;

// Enumeration

- (NSEnumerator *)lineEnumerator;

// NGTextOutputStream, NGExtendedTextOutputStream

- (void)writeCharacter:(unichar)_character;
- (void)writeString:(NSString *)_string;
- (void)flush;

- (void)writeNewline;

@end

#endif /* __NGStreams_NGCTextStream_H__ */
