/*
   NGFileStream.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGFileStream.h,v 1.14 1999/03/26 12:29:53 helge Exp $

#ifndef __NGStreams_NGFileStream_H__
#define __NGStreams_NGFileStream_H__

#ifdef HAVE_WINDOWS_H
#  include <windows.h>
#endif
#ifdef HAVE_POLL_H
#  include <poll.h>
#endif
#ifdef HAVE_SYS_POLL_H
#  include <sys/poll.h>
#endif
#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif

#if defined(WIN32)
#  include <winsock.h>
#endif

#include <NGStreams/NGStream.h>
#include <NGStreams/NGStreamProtocols.h>
#include <NGStreams/NGUrl.h>

@class NSFileHandle;
@class NGFileUrl;

extern NSString *NGFileReadOnly;
extern NSString *NGFileWriteOnly;
extern NSString *NGFileReadWrite;
extern NSString *NGFileAppend;
extern NSString *NGFileReadAppend;

extern id<NGInputStream>  NGIn;
extern id<NGOutputStream> NGOut;
extern id<NGOutputStream> NGErr;
extern void NGInitStdio(void);

@interface NGFileStream : NGStream < NGPositionableStream >
{
@private
#if defined(WIN32)
  HANDLE fh; // Windows file handle
#else
  int    fd; // Unix file descriptor
#endif
  
  NGStreamMode streamMode;
  NSString     *systemPath;
  NSFileHandle *handle;     // not retained !

  unsigned markDelta; // tracks mark, for marking
}

+ (id)streamForResource:(id<NGUrl>)_url;
- (id)initWithFileUrl:(NGFileUrl *)_url;
- (id)initWithFileHandle:(NSFileHandle *)_handle; // use with care !

// throws
//   NGUnknownStreamModeException  when _mode is invalid
//   NGCouldNotOpenStreamException when the file could not be opened
- (void)openInMode:(NSString *)_mode;

- (BOOL)isOpen;

// Foundation file handles

- (NSFileHandle *)fileHandle;
- (int)fileDescriptor;

#if defined(WIN32)
- (HANDLE)windowsFileHandle;
#endif

// primitives

// throws
//   NGWriteOnlyStreamException when the stream is not readable
//   NGStreamNotOpenException   when the stream is not open
//   NGEndOfStreamException     when the end of the stream is reached
//   NGReadErrorException       when the read call failed
- (unsigned)readBytes:(void *)_buf count:(unsigned)_len;

// throws
//   NGReadOnlyStreamException when the stream is not writeable
//   NGStreamNotOpenException  when the stream is not open
//   NGWriteErrorException     when the write call failed
- (unsigned)writeBytes:(const void *)_buf count:(unsigned)_len;

// throws NGCouldNotCloseStreamException when the close call failed
- (void)close;

- (NGStreamMode)mode;
- (BOOL)isRootStream;

// blocking

- (BOOL)wouldBlockInMode:(NGStreamMode)_mode; // not supported with Windows

// marking

- (void)mark;
- (void)rewind;
- (BOOL)markSupported; // returns YES

// NGPositionableStream

// throws
//   NGStreamSeekErrorException
- (void)moveToLocation:(unsigned)_location; // note that absolute moves delete marks

// throws
//   NGStreamSeekErrorException
- (void)moveByOffset:(int)_delta;

@end

#endif /* __NGStreams_NGFileStream_H__ */
