/* 
   NGFileUrl.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGFileUrl.h,v 1.17 1999/01/27 16:41:14 helge Exp $

#ifndef __NGStreams_NGFileUrl_H__
#define __NGStreams_NGFileUrl_H__

#include <NGStreams/NGUrl.h>
#include <NGStreams/NGStream.h>

@class NSString, NSDate;

extern NSString *NGFileUrlProtocolName;

typedef BOOL (*NGChildNameFilterFunction)(id<NGTreeUrl> _parent, NSString *_name,
                                          id _context);
typedef int  (*NGCompareUrlFunction)(id<NGUrl> _url1, id<NGUrl> _url2,
                                     id _context);

@interface NGFileUrl : NGUrl < NSCopying, NSCoding, NGTreeUrl >
{
@private
  NSString *path;
  NSString *systemPath; // path in system encoding
}

+ (id)urlWithString:(NSString *)_string
  protocol:(NSString *)_protocolName;
+ (id)fileUrlWithString:(NSString *)_string;
+ (id)fileUrlWithPath:(NSString *)_path;
+ (id)fileUrlForCurrentDirectory;

- (id)initWithString:(NSString *)_string
  protocol:(NSString *)_protocolName;

// accessors

- (NSString *)path;            // return path in URL notation
- (NSString *)fileSystemPath;  // return path in OS based notation
- (NSString *)resourceName;

- (id<NGUrl>)parentResource;
- (id<NGUrl>)childResourceWithName:(NSString *)_name;
- (id<NGUrl>)childResourceWithName:(NSString *)_name extension:(NSString *)_ext;

// operations

- (BOOL)doesResourceExist;
- (BOOL)isAccessibleInMode:(NGStreamMode)_mode;

// stat info

- (NSDictionary *)attributes;     // attributes defined by NSFileManager
- (unsigned long long)size;       // returns the size of the resource in bytes
- (NSDate *)lastModificationDate; // time of last modification (write)

// NGUrl

- (NSString *)protocol;
- (NSString *)stringValue;

// NGTreeUrl

- (NSString *)pathRelativeToParent:(NGFileUrl *)_parent;

- (BOOL)isLeafResource;           // is resource a file ?
- (BOOL)isCollection;             // is resource a directory ?
- (NSEnumerator *)childResources; // throws NGInvalidUrlException

- (NSEnumerator *)childResourcesWithFilter:(NGChildNameFilterFunction)_filter
  sortUsingFunction:(NGCompareUrlFunction)_compare
  context:(id)_userData;
- (NSEnumerator *)childResourcesWithExtension:(NSString *)_extension;

#if HAVE_NGRegularExpression
- (NSEnumerator *)childResourcesWithPattern:(NSString *)_pattern;
#endif

// creates new directory
// throws
//   NGUrlCollectionAlreadyExistsException
//     if there is already a subresource with the name
//   NGUrlCouldNotCreateCollectionException
//     if the collection could not be created
- (id<NGTreeUrl>)createCollectionWithName:(NSString *)_name;

// operations

// throws
//   NGUrlCouldNotDeleteResourceException  when the resource couldn't be deleted
- (void)deleteResource;

@end

#import <Foundation/NSProcessInfo.h>

@interface NSProcessInfo(NGCurrentDirectory)

- (NGFileUrl *)currentResource; // throws NGIOAccessException, NGIOException

@end

#import <Foundation/NSFileManager.h>

@interface NSFileManager(NGFileUrlExtensions)

- (NGFileUrl *)currentDirectoryResource;

@end

#import <Foundation/NSTask.h>

@interface NSTask(NGFileUrlExtensions)

- (void)setCurrentDirectoryResource:(NGFileUrl *)_url;
- (NGFileUrl *)currentDirectoryResource;
- (void)setLaunchUrl:(NGFileUrl *)_url;
- (NGFileUrl *)launchUrl;

@end

#import <Foundation/NSBundle.h>

@interface NSBundle(NGFileUrlExtensions)

- (NGFileUrl *)resourceWithName:(NSString *)_name ofType:(NSString *)_type;

- (NGFileUrl *)resourceWithName:(NSString *)_name ofType:(NSString *)_type
  inDirectory:(NGFileUrl *)_dirResource;

- (NGFileUrl *)bundleResource;

@end

#endif /* __NGStreams_NGFileUrl_H__ */
