/* 
   NGFilterStream.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGFilterStream.m,v 1.8 1999/01/27 16:41:14 helge Exp $

#include "common.h"
#include "NGFilterStream.h"

@implementation NGFilterStream

+ (id)filterWithInputSource:(id<NGInputStream>)_source {
  return AUTORELEASE([[self alloc] initWithInputSource:_source]);
}
+ (id)filterWithOutputSource:(id<NGOutputStream>)_source {
  return AUTORELEASE([[self alloc] initWithOutputSource:_source]);
}
+ (id)filterWithSource:(id<NGStream>)_source {
  return AUTORELEASE([[self alloc] initWithSource:_source]);
}

- (id)init {
  return [self initWithSource:nil];
}

- (id)initWithSource:(id<NGStream>)_source {
  if ((self = [super init])) {
    self->source = RETAIN(_source);

    if ([source isKindOfClass:[NSObject class]]) {
      self->readBytes  = (NGIOReadMethodType)
        [(NSObject *)self->source methodForSelector:@selector(readBytes:count:)];
      self->writeBytes = (NGIOWriteMethodType)
        [(NSObject *)self->source methodForSelector:@selector(writeBytes:count:)];
    }
  }
  return self;
}

- (id)initWithInputSource:(id<NGInputStream>)_source {
  if ((self = [super init])) {
    self->source = RETAIN(_source);

    if ([source isKindOfClass:[NSObject class]]) {
      self->readBytes  = (NGIOReadMethodType)
        [(NSObject *)self->source methodForSelector:@selector(readBytes:count:)];
    }
  }
  return self;
}
- (id)initWithOutputSource:(id<NGOutputStream>)_source {
  if ((self = [super init])) {
    self->source = RETAIN(_source);

    if ([source isKindOfClass:[NSObject class]]) {
      self->writeBytes  = (NGIOWriteMethodType)
        [(NSObject *)self->source methodForSelector:@selector(writeBytes:count:)];
    }
  }
  return self;
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->source);
  self->readBytes  = NULL;
  self->writeBytes = NULL;
  [super dealloc];
}
#endif

// accessors

- (id<NGInputStream>)inputStream {
  return [self source];
}
- (id<NGOutputStream>)outputStream {
  return [self source];
}
- (id<NGStream>)source {
  return self->source;
}

// primitives

- (unsigned)readBytes:(void *)_buf count:(unsigned)_len {
  if (self->readBytes)
    return (unsigned)readBytes(self->source, _cmd, _buf, _len);
  else
    return [self->source readBytes:_buf count:_len];
}
- (unsigned)writeBytes:(const void *)_buf count:(unsigned)_len {
  if (self->writeBytes)
    return (unsigned)writeBytes(self->source, _cmd, _buf, _len);
  else
    return [self->source writeBytes:_buf count:_len];
}

- (void)flush {
  [self->source flush];
}
- (void)close {
  [self->source close];
}

- (NGStreamMode)mode {
  return [self->source mode];
}
- (BOOL)isRootStream {
  return NO;
}

// all other things are forward

- (void)forwardInvocation:(NSInvocation *)_invocation {
  if ([self->source respondsToSelector:[_invocation selector]]) {
    [_invocation setTarget:self->source];
    [_invocation invoke];
  }
  else
    [self doesNotRecognizeSelector:[_invocation selector]];
}

// description

- (NSString *)description {
  return [NSString stringWithFormat:
                     @"<%@[0x%08X] source=%@ mode=%@>",
                     NSStringFromClass([self class]), (unsigned)self,
                     self->source ? (id)self->source : (id)@"nil",
                     [self modeDescription]];
}

@end
