/* 
   NGRelativeUrl.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGRelativeUrl.m,v 1.5 1999/01/27 16:41:16 helge Exp $

#include "common.h"
#include "NGRelativeUrl.h"
#include "NGUrlExceptions.h"

@implementation NGRelativeUrl

+ (id)urlWithString:(NSString *)_name relativeToUrl:(id<NGTreeUrl>)_url {
  return AUTORELEASE([[self alloc] initWithString:_name relativeToUrl:_url]);
}

- (id)initWithString:(NSString *)_name relativeToUrl:(id<NGTreeUrl>)_url {
  if ((self = [super init])) {
    self->parentUrl    = RETAIN(_url);
    self->resourceName = [_name copyWithZone:[self zone]];
  }
  return self;
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->parentUrl);    self->parentUrl    = nil;
  RELEASE(self->resourceName); self->resourceName = nil;
  [super dealloc];
}
#endif

// accessors

- (id<NGUrl>)parentResource {
  return self->parentUrl;
}

- (NSString *)resourceName {
  return self->resourceName;
}

// NGUrl

- (NSString *)protocol {
  return [self->parentUrl protocol];
}

- (NSString *)stringValue {
  NSMutableString *str = [self->parentUrl stringValue];
  [str appendString:@"/"];
  [str appendString:resourceName];
  return AUTORELEASE(str);
}

// NGTreeUrl

- (BOOL)isLeafResource {
  return YES;
}
- (BOOL)isCollection {
  return NO;
}
- (NSEnumerator *)childResources {
  return nil;
}

- (id<NGUrl>)produceRelativeUrlForChildResource:(NSString *)_path {
  THROW([[NGInvalidUrlTypeException alloc]
            initWithReason:@"tried to produce child-url for leaf-resource .."]);
  return nil;
}

@end
