/* 
   NGStream+serialization.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGStream+serialization.h,v 1.3 1999/01/27 15:47:03 helge Exp $

#ifndef __NGStreams_NGStream_serialization_H__
#define __NGStreams_NGStream_serialization_H__

#import <Foundation/NSSerialization.h>
#include <NGStreams/NGStream.h>
#include <NGStreams/NGStreamProtocols.h>

/*
  Serialization is implemented as a category because of it's importance. From
  a design point of view it would be better placed in an extra class, so that
  the serialization scheme could be replaced.
*/

@interface NGStream(serialization) < NGSerializer >

- (void)serializeChar:(char)_value;
- (void)serializeShort:(short)_value;
- (void)serializeInt:(int)_value;
- (void)serializeLong:(long)_value;
- (void)serializeFloat:(float)_value;
- (void)serializeDouble:(double)_value;
- (void)serializeLongLong:(long long)_value;

- (char)deserializeChar;
- (short)deserializeShort;
- (int)deserializeInt;
- (long)deserializeLong;
- (float)deserializeFloat;
- (double)deserializeDouble;
- (long long)deserializeLongLong;

- (void)serializeCString:(const char *)_value;
- (char *)deserializeCString;

- (void)serializeDataAt:(const void*)data ofObjCType:(const char*)type
  context:(id<NSObjCTypeSerializationCallBack>)_callback;
- (void)deserializeDataAt:(void *)data ofObjCType:(const char*)type
  context:(id<NSObjCTypeSerializationCallBack>)_callback;

@end

void NGStreamSerializeObjC(id<NGStream> self,
                           const void *_data, const char *_type,
                           id<NSObjCTypeSerializationCallBack> _callback);

void NGStreamDeserializeObjC(id<NGStream> self,
                             void *_data, const char *_type,
                             id<NSObjCTypeSerializationCallBack> _callback);

#endif /* __NGStreams_NGStream_serialization_H__ */
