/* 
   NGStream.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGStream.h,v 1.4 1999/01/27 15:47:03 helge Exp $

#ifndef __NGStreams_NGStream_H__
#define __NGStreams_NGStream_H__

#import <Foundation/NSObject.h>
#include <NGStreams/NGStreamProtocols.h>

static inline BOOL NGCanReadInStreamMode(NGStreamMode _mode) {
  return ((_mode == NGStreamMode_readOnly) || (_mode == NGStreamMode_readWrite));
}
static inline BOOL NGCanWriteInStreamMode(NGStreamMode _mode) {
  return ((_mode == NGStreamMode_writeOnly) || (_mode == NGStreamMode_readWrite));
}

@interface NGStream : NSObject < NGStream, NGByteSequenceStream >
{
}

// ******************** primitives ********************

// Never returns 0. If an EOF like condition occures, NGEndOfStreamException
// is thrown.
- (unsigned)readBytes:(void *)_buf count:(unsigned)_len;       // abstract
- (unsigned)writeBytes:(const void *)_buf count:(unsigned)_len; // abstract

- (void)flush; // empty
- (void)close; // empty

- (NGStreamMode)mode; // abstract
- (BOOL)isRootStream; // abstract

// methods method which write exactly _len bytes

- (void)safeReadBytes:(void *)_buf count:(unsigned)_len;
- (void)safeWriteBytes:(const void *)_buf count:(unsigned)_len;

// marking

- (void)mark;           // does nothing
- (void)rewind;         // does nothing
- (BOOL)markSupported;  // returns NO

// convenience methods

- (int)readByte; // java semantics (-1 returned on EOF)

// description

- (NSString *)modeDescription;

@end

// concrete implementations as functions (to be used in non-subclasses)

extern int NGReadByteFromStream(id<NGInputStream> _stream);

extern void
NGSafeReadBytesFromStream(id<NGInputStream> _in, void *_buf, unsigned _len);

extern void
NGSafeWriteBytesToStream(id<NGOutputStream> _out,const void *_buf,unsigned _len);

#endif /* __NGStreams_NGStream_H__ */
