/* 
   NGStringTextStream.m
   
   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGStringTextStream.m,v 1.4 1999/01/27 16:41:17 helge Exp $

#include "common.h"
#include "NGStringTextStream.h"
#include "NGStreamExceptions.h"

@implementation NGStringTextStream

+ (id)textStreamWithString:(NSString *)_string {
  return AUTORELEASE([[self alloc] initWithString:_string]);
}
- (id)initWithString:(NSString *)_string {
  if ((self = [super init])) {
    string    = RETAIN(_string);
    index     = 0;
    isMutable = [_string isKindOfClass:[NSMutableString class]];
  }
  return self;
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(string); string = nil;
  [super dealloc];
}
#endif

// accessors

- (NSString *)string {
  return string;
}

// operations

- (void)close {
  // releases string
  RELEASE(string); string = nil;
}

// NGTextInputStream

- (unichar)readCharacter {
  // throws
  //   NGStreamNotOpenException  when the stream is not open
  
  unsigned currentLength = [string length];
  unichar  result;

  if (string == nil) {
    THROW([[NGStreamNotOpenException alloc] initWithReason:
            @"tried to read from a string text stream which was closed"]);
  }
  
  if (currentLength == index)
    THROW([[NGEndOfStreamException alloc] init]);

  result = [string characterAtIndex:index];
  index++;
  return result;
}

// NGExtendedTextInputStream

- (NSString *)readLineAsString {
  // throws
  //   NGStreamNotOpenException  when the stream is not open
  
  unsigned currentLength = [string length];
  NSRange  range;
  
  if (string == nil) {
    THROW([[NGStreamNotOpenException alloc] initWithReason:
            @"tried to read from a string text stream which was closed"]);
  }
  
  if (currentLength == index)
    //THROW([[NGEndOfStreamException alloc] init]);
    return nil;
  
  range.location = index;
  range.length   = (currentLength - index);

  range = [string rangeOfString:@"\n" options:NSLiteralSearch range:range];
  if (range.length == 0) { // did not found newline
    NSString *result = [string substringFromIndex:index];
    index = currentLength;
    return result;
  }
  else {
    NSString *result = nil;

    range.length   = (range.location - index);
    range.location = index;

    result = [string substringWithRange:range];

    index += range.length + 1;

    return result;
  }
}

// NGTextOutputStream

- (void)writeCharacter:(unichar)_character {
  // throws
  //   NGReadOnlyStreamException when the stream is not writeable
  //   NGStreamNotOpenException  when the stream is not open
  
  if (string == nil) {
    THROW([[NGStreamNotOpenException alloc] initWithReason:
            @"tried to write to a string text stream which was closed"]);
  }
  if (!isMutable)
    THROW([[NGReadOnlyStreamException alloc] init]);
  
  [(NSMutableString *)string appendString:
    [NSString stringWithCharacters:&_character length:1]];
}

- (void)writeString:(NSString *)_string {
  // throws
  //   NGReadOnlyStreamException when the stream is not writeable
  //   NGStreamNotOpenException  when the stream is not open
  
  if (string == nil) {
    THROW([[NGStreamNotOpenException alloc] initWithReason:
            @"tried to write to a string text stream which was closed"]);
  }
  if (!isMutable)
    THROW([[NGReadOnlyStreamException alloc] init]);
  
  [(NSMutableString *)string appendString:_string];
}

- (void)flush {
}

@end
