/* 
   NGTextStream.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGTextStream.m,v 1.7 1999/01/27 16:41:18 helge Exp $

#include "common.h"
#include "NGTextStream.h"

@implementation NGTextStream

// NGTextInputStream

- (unichar)readCharacter {
  [self subclassResponsibility:_cmd];
  return 0;
}

// NGExtendedTextInputStream

- (NSString *)readLineAsString {
  NSMutableString *str = [NSMutableString string];
  unichar c;

  NS_DURING {
    while ((c = [self readCharacter]) != '\n')
      [str appendString:[NSString stringWithCharacters:&c length:1]];
  }
  NS_HANDLER {
    if ([localException isKindOfClass:[NGEndOfStreamException class]]) {
      if ([str length] == 0) str = nil;
    }
    else
      [localException raise];
  }
  NS_ENDHANDLER;
  
  return str;
}

// NGTextOutputStream

- (void)writeCharacter:(unichar)_character {
  [self subclassResponsibility:_cmd];
}

- (void)writeString:(NSString *)_string {
  unsigned length = [_string length], cnt = 0;
  unichar  buffer[length];
  void     (*writeChar)(id, SEL, unichar);

  writeChar = (void (*)(id,SEL,unichar))
    [self methodForSelector:@selector(writeCharacter:)];
  
  [_string getCharacters:buffer];
  for (cnt = 0; cnt < length; cnt++)
    writeChar(self, @selector(writeCharacter:), buffer[cnt]);
}

- (void)flush {
}

// NGExtendedTextOutputStream

- (void)writeFormat:(NSString *)_format arguments:(va_list)_ap {
  NSString *tmp = [[NSString alloc] initWithFormat:_format arguments:_ap];
  AUTORELEASE(tmp);
  [self writeString:tmp];
}
- (void)writeFormat:(NSString *)_format, ... {
  va_list ap;

  va_start(ap, _format);

  NS_DURING {
    [self writeFormat:_format arguments:ap];
  }
  NS_HANDLER {
    va_end(ap);
    [localException raise];
  }
  NS_ENDHANDLER;
  va_end(ap);
}

- (void)writeNewline {
  [self writeString:@"\n"];
  [self flush];
}

@end
