/* 
   NGUrl.h

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGUrl.h,v 1.12 1999/01/27 16:11:17 helge Exp $

#ifndef __NGStreams_NGUrl_H__
#define __NGStreams_NGUrl_H__

#import <Foundation/NSObject.h>

@class NSString;

@protocol NGUrl < NSObject >

- (NSString *)protocol;
- (NSString *)stringValue;
- (NSString *)resourceName;

@end

@protocol NGTreeUrl < NGUrl >

- (BOOL)isLeafResource;
- (NSEnumerator *)childResources;
- (id<NGUrl>)produceRelativeUrlForChildResource:(NSString *)_path; // retained result

@end

@protocol NGUrlFactory < NSObject >

- (id<NGUrl>)produceUrlForString:(NSString *)_string
  protocol:(NSString *)_protocolName;

@end

@interface NGUrl : NSObject < NGUrl >
{
}

// URL factory

+ (id<NGUrl>)urlWithString:(NSString *)_url; // throws NGInvalidUrlException

+ (void)registerFactory:(id<NGUrlFactory>)_factory
  forProtocolWithName:(NSString *)_protocol;

// NGUrl

- (NSString *)protocol;     // abstract
- (NSString *)stringValue;  // abstract
- (NSString *)resourceName; // abstract

@end

/*
  These functions encode/decode HTTP style URL paths, which can escape
  spaces and special chars.
  Chars are escaped using the '%' hex-notation:

  Encode:
      'Hello World' => 'Hello%20World'
      '& ?' => '%26%20%3F'
*/
BOOL     NGContainsUrlEscapeCharacters(const char *_buffer);
void     NGUnescapeUrlBuffer(const char *_source, char *_dest);
NSString *NGUnescapeUrlString(NSString *_source);
void     NGEscapeUrlBuffer(const char *_source, char *_dest);
NSString *NGEscapeUrlString(NSString *_source);

#endif /* __NGStreams_NGUrl_H__ */
