// $Id: NSData+gzip.m,v 1.4 1999/01/16 16:19:05 helge Exp $

#include "zutil.h"
#undef Assert
#import "common.h"
#import "NSData+gzip.h"

@implementation NSData(gzip)

- (NSData *)gzip {
  return [self gzipWithLevel:Z_DEFAULT_COMPRESSION];
}

- (NSData *)gzipWithLevel:(int)_level {
  NSMutableData *data     = nil;
  int           errorCode = 0;
  unsigned      len       = [self length];
  void          *src      = (void *)[self bytes];
  IMP           addBytes  = NULL;
  char          outBuf[4096];
  z_stream      out;
  uLong         crc;

  static inline void putLong(uLong x) {
    int n;
    for (n = 0; n < 4; n++) {
      unsigned char c = (int)(x & 0xff);
      addBytes(data, @selector(appendBytes:length:), &c, 1);
      x >>= 8;
    }
  }
  
  NSAssert1((_level >= NGGZipMinimalCompression &&
             _level <= NGGZipMaximalCompression)
            || (_level == Z_DEFAULT_COMPRESSION),
            @"invalid compression level %i (0-9)", _level);

  data = [NSMutableData dataWithCapacity:
                          (len / 10 < 128) ? len : len / 10];
  addBytes = [data methodForSelector:@selector(appendBytes:length:)];

  out.zalloc    = (alloc_func)NULL;
  out.zfree     = (free_func)NULL;
  out.opaque    = (voidpf)NULL;
  out.next_out  = (Byte*)&outBuf;
  out.avail_out = sizeof(outBuf);
  out.next_in   = Z_NULL;
  out.avail_in  = 0;
  errorCode     = Z_OK;
  crc           = crc32(0L, Z_NULL, 0);

  errorCode = deflateInit2(&out, _level, Z_DEFLATED, -MAX_WBITS,
                           DEF_MEM_LEVEL,
                           0); // windowBits is passed <0 to suppress zlib header
  if (errorCode != Z_OK) {
    NSLog(@"ERROR: could not init deflate !");
    return nil;
  }

  { // add gzip header
    char buf[10] = {
      0x1f, 0x8b,    // magic
      Z_DEFLATED, 0, // flags
      0, 0, 0, 0,    // time
      0, OS_CODE     // flags
    };
    addBytes(data, @selector(appendBytes:length:), &buf, 10);
  }
  
  { // gz_write
    out.next_in  = src;
    out.avail_in = len;
    
    while (out.avail_in > 0) {
      if (out.avail_out == 0) {
        out.next_out = (void *)&outBuf; // reset buffer position
        addBytes(data, @selector(appendBytes:length:), &outBuf, sizeof(outBuf));
        out.avail_out = sizeof(outBuf);
      }
      errorCode = deflate(&out, Z_NO_FLUSH);
      if (errorCode != Z_OK) {
        NSLog(@"ERROR: could not deflate chunk !");
        if (out.state) deflateEnd(&out);
        return nil;
      }
    }
    crc = crc32(crc, src, len);
  }

  { // gz_flush
    BOOL done = NO;
    
    out.next_in  = NULL;
    out.avail_in = 0; // should be zero already anyway
    
    for (;;) {
      len = sizeof(outBuf) - out.avail_out;

      if (len > 0) {
        addBytes(data, @selector(appendBytes:length:), &outBuf, len);
        out.next_out  = (void *)&outBuf;
        out.avail_out = sizeof(outBuf);
      }
      if (done)
        break;
      errorCode = deflate(&out, Z_FINISH);

      // deflate has finished flushing only when it hasn't used up
      // all the available space in the output buffer: 
      done = (out.avail_out != 0 || errorCode == Z_STREAM_END);

      if (errorCode != Z_OK && errorCode != Z_STREAM_END)
        break;
    }
    if (errorCode != Z_STREAM_END) {
      NSLog(@"ERROR: flush failed.");
      if (out.state) deflateEnd(&out);
      return nil;
    }
  }
  { // write trailer (checksum and filesize)
    putLong(crc);
    putLong(out.total_in);
  }
  if (out.state) deflateEnd(&out);

  return data;
}

@end

void __link_NSData_gzip(void) {
  __link_NSData_gzip();
}
