
#import <Foundation/Foundation.h>
#import <NGExtensions/NGExtensions.h>
#import <NGStreams/NGStreams.h>
#import <NGNet/NGNet.h>

static void testInetSocket(void) {
  NGInternetSocketAddress *address;
  NGActiveSocket          *socket;

  address = [NGInternetSocketAddress addressWithPort:80 onHost:@"www.mdlink.de"];
  NSLog(@"Inet: %@", address);

  socket = [NGActiveSocket socketInDomain:[address domain]];
  NSLog(@"Socket: %@", socket);

  [socket connectToAddress:address];
  NSLog(@"Socket: %@", socket);

  TRY {
    NGTextStream *io = [NGCTextStream textStreamWithSource:socket];
    NSString *line;
    unsigned cnt = 0;

    NSLog(@"Writing GET / ..");
    [io writeString:@"GET /home/index.html HTTP/1.0\r\n\r\n"];
    [io flush];

    do {
      line = [io readLineAsString];
      cnt++;
      [NGTextOut writeFormat:@"line %d|%@", cnt, line];
      [NGTextOut writeNewline];
    }
    while (line);
  } END_TRY;
  CATCH (NGIOException) {
    NSLog(@"IO: %@", localException);
  }
  OTHERWISE {
    NSLog(@"Exception: %@", localException);
  }
  END_CATCH;

  [socket shutdown];
}

static void testBinaryFiles(void) {
  NGFileUrl        *url;
  NGFileStream     *in;

  url   = [NGFileUrl fileUrlWithPath:
              @"H:\\dev\\MOF2\\Foundation\\tests\\TestStreams\\TestStreams_main.m"];
  in    = [NGFileStream streamForResource:url];
  [in openInMode:NGFileReadOnly];

  {
    unsigned size = [url size];
    char *buffer;

    NSLog(@"size of %@: %d bytes", url, size);
    buffer = malloc(size);
    [in safeReadBytes:buffer count:size];
    NSLog(@"data is %@", [NSData dataWithBytesNoCopy:buffer length:size]);
  }
  [in close];
}

static void testTextFiles(void) {
  NGFileUrl        *url;
  NGFileStream     *in;
  NGBufferedStream *bufin;
  NGCTextStream    *txtIn;

  url   = [NGFileUrl fileUrlWithPath:
              @"H:\\dev\\MOF2\\Foundation\\tests\\TestStreams\\TestStreams_main.m"];
  in    = [NGFileStream streamForResource:url];
  [in openInMode:NGFileReadOnly];
  bufin = [NGBufferedStream filterWithSource:in];
  txtIn = [NGCTextStream textStreamWithSource:bufin];

  {
    NSString *line;
    int count = 0;

    while ((line = [txtIn readLineAsString])) {
      count++;
      if (count % 10 == 0)
        NSLog(@"line[%i]: %@", count, line);
    }
    NSLog(@"%i lines", count);
  }
  [txtIn close];
}

static void tryWindowsStdio(void) {
  HANDLE io;
  DWORD  written;
  DWORD  write = 6;

  io = CreateFile("CONOUT$", GENERIC_WRITE,
                  FILE_SHARE_WRITE,
                  NULL,
                  OPEN_EXISTING,
                  0,
                  NULL);
  if (io == INVALID_HANDLE_VALUE)
    NSLog(@"could not create CONOUT$ ..");

  NSLog(@"opened CONIN ..");
  
  if (!WriteConsole(io, "1234\r\n", write, &written, NULL)) {
    NSLog(@"write failed %i, written=%i", GetLastError(), written);
  }
  else {
    NSLog(@"did write %i", written);
  }
}

void main (int argc, const char *argv[])
{
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
  {
    NGInitTextStdio();
    [NGTextOut writeString:@"running MOF2 tool..\n"];
    [NGTextOut flush];
    
    //testTextFiles();
    //testBinaryFiles();
    testInetSocket();
  }
  [pool release];
  exit(0);       // insure the process exit status is 0
  __MissingMethods_dummy_function_for_stupid_linkers();
}

LINK_NGExtension;
LINK_NGStreams;
