/* 
   coder.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: coder.m,v 1.12 1999/01/21 13:24:48 helge Exp $

#import <NGStreams/NGStreamCoder.h>
#import "common.h"

#define COUNT     1000

void _timeFD(id dict) {
  struct timeval tv;
  NSTimeInterval t;
  Class ArchiverClass   = [NSArchiver class];
  Class UnarchiverClass = [NSUnarchiver class];

  gettimeofday(&tv, NULL);
  t = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0);
  {
    IMP archive;
    IMP unarchive;
    int pos;

    archive   = [ArchiverClass   methodForSelector:
                                   @selector(archivedDataWithRootObject:)];
    unarchive = [UnarchiverClass methodForSelector:
                                   @selector(unarchiveObjectWithData:)];

    for (pos = 0; pos < COUNT; pos++) {
      id data, obj;

      data= archive(ArchiverClass, @selector(archivedDataWithRootObject:), dict);
      obj= unarchive(UnarchiverClass, @selector(unarchiveObjectWithData:), data);
      
      if (obj == nil) abort();
    }
  }
  gettimeofday(&tv, NULL);
  t = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0) - t;

  NSLog(@"FD took %4.3fs.", t < 0.0 ? -1.0 : t);
}

NGOptionDescription options[] = {
  { @"-?",   @"--help",     NGHelpOption,     0, NULL, @"show help page" },
  NGLogOption,
  NGLastOption
};

NGMainOpt(options) {
  NGInitTextStdio();
  [NSAutoreleasePool enableDoubleReleaseCheck:NO];

  {
    NSData       *data = [NSMutableData dataWithCapacity:512];
    NSArchiver   *archiver;
    NSUnarchiver *unarchiver;
    id           hello = @"hello", h1, h2;

    archiver = [[NSArchiver alloc]
                            initForWritingWithMutableData:(NSMutableData *)data];
    AUTORELEASE(archiver);

    [archiver encodeObject:hello];
    [archiver encodeObject:hello];

    data = [archiver archiverData];
    unarchiver = [[NSUnarchiver alloc] initForReadingWithData:data];
    h1 = [unarchiver decodeObject];
    h2 = [unarchiver decodeObject];
    if (h1 != h2) NSLog(@"invalid decoding ..");
    RELEASE(unarchiver);

    archiver = nil;
  }
  
  {
    NSData *data;
    id     tmp;
    id     result;

    data   = [NSArchiver archivedDataWithRootObject:@"hello"];
    result = [NSUnarchiver unarchiveObjectWithData:data];

    if (![result isEqualToString:@"hello"])
      NSLog(@"Result %@ != hello", result);
    else
      NSLog(@"Result %@ == hello", result);

    tmp = data;
    data   = [NSArchiver archivedDataWithRootObject:tmp];
    result = [NSUnarchiver unarchiveObjectWithData:data];

    if (![result isEqual:tmp])
      NSLog(@"Result %@ != %@", result, tmp);
  }

  {
    id dict  = [NSMutableDictionary dictionaryWithCapacity:256];
    id hello = @"hello";

    [dict setObject:[NSArray arrayWithObjects:@"1", intObj(2), hello, nil]
          forKey:@"array"];
    [dict setObject:hello forKey:@"world"];
    [dict setObject:hello forKey:@"world2"];
    [dict setObject:[NSArray array]      forKey:@"empty array"];
    [dict setObject:[NSDictionary dictionary] forKey:@"empty dictionary"];
    [dict setObject:[NSMutableArray array]        forKey:@"empty mutable array"];
    [dict setObject:[NSMutableDictionary dictionary]
          forKey:@"empty mutable dict"];
    [dict setObject:[NSData data]                 forKey:@"empty data"];
    [dict setObject:[NSCalendarDate calendarDate] forKey:@"calendarDate"];
    [dict setObject:[NSDate date]                 forKey:@"date"];
    [dict setObject:[NSData dataWithBytes:"12345" length:5] forKey:@"data"];

    NSLog(@"dict = %@", dict);

    {
      NSAutoreleasePool *pool = nil;

      pool = [NSAutoreleasePool new];
      _timeFD(dict);
      [pool release];
    }

    {
      NGDataStream  *data  = nil;
      NGStreamCoder *coder = nil;
      NSData        *fData = nil;

      fData = [NSArchiver archivedDataWithRootObject:dict];
      
      coder = [NGStreamCoder coderWithStream:(data=[NGDataStream dataStream])];
      [coder encodeRootObject:dict];

      NSLog(@"resulting data is %i bytes (foundation=%i, equal=%s).",
            [[data data] length], [fData length],
            [[data data] isEqual:fData] ? "YES" : "NO");

      [data moveToLocation:0];

      NSLog(@"unarchive from data ..");
      {
        id result = [[NSUnarchiver unarchiveObjectWithData:[data data]] retain];

        NSLog(@"decoded = %@, equal=%s",
              result, [result isEqual:dict] ? "yes" : "no");
        
        RELEASE(result);
      }

      NSLog(@"unarchive from stream..");
      {
        id result = RETAIN([coder decodeObject]);
        
        NSLog(@"decoded = %@, equal=%s",
              result, [result isEqual:dict] ? "yes" : "no");
        
        RELEASE(result);
      }
    }
  }
}
NGEndMain;

LINK_NGExtensions;
LINK_NGStreams;
