/* 
   do.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: do.m,v 1.10 1999/01/21 14:17:02 helge Exp $

#import "common.h"

@protocol Calculator

- (void)add:(int)_value;
- (oneway void)subtract:(int)_value;
- (void)multiplyBy:(int)_value;
- (void)devideBy:(int)_value;

- (int)value;
- (void)addValueToMutableString:(NSMutableString *)_string;
- (void)throwException;

@end

@interface Calculator : NSObject < Calculator >
{
  int value;
}

@end

@implementation Calculator

/*
- (NSMethodSignature *)methodSignatureForSelector:(SEL)_selector {
  NSMethodSignature *signature = [super methodSignatureForSelector:_selector];
  
  NSLog(@"calc was asked for signature of '%@'%s",
        NSStringFromSelector(_selector),
        [signature isOneway] ? ", it is oneway." : ".");

  {
    struct objc_method *mth;

    mth = class_get_instance_method(*(Class *)self, _selector);
    
    if (mth) {
      NSLog(@"  found signature '%s' ..", mth->method_types);
    }
  }
  
  return signature;
}
*/

- (int)value {
  return self->value;
}

- (void)add:(int)_value {
  NSLog(@"add: %i", _value);
  self->value += _value;
}
- (oneway void)subtract:(int)_value {
  NSLog(@"subtract: %i", _value);
  self->value -= _value;
}
- (void)multiplyBy:(int)_value {
  NSLog(@"multiply: %i", _value);
  self->value *= _value;
}
- (void)devideBy:(int)_value {
  NSLog(@"devide: %i", _value);
  self->value /= _value;
}

- (void)addValueToMutableString:(NSMutableString *)_string {
  [_string appendString:[NSString stringWithFormat:@"%i", [self value]]];
}

- (void)throwException {
  THROW([[NSException alloc] initWithReason:@"Hallo Welt !"]);
}

@end

int port = 0;

static void client(id<NGSocketAddress> _address) {
  NSAutoreleasePool       *pool = [NSAutoreleasePool new];
  NGConnection            *connection = nil;
  id<NSObject,Calculator> calculator  = nil;

  connection = [NGConnection connectionWithRegisteredAddress:_address];
  if (connection == nil) {
    NSLog(@"could not connect ..");
    return;
  }
  
  calculator = (Calculator *)[[connection rootProxy] retain];
  NSLog(@"got root object '%@' ..", calculator);

  [calculator add:3];
  [calculator multiplyBy:2];
  [calculator subtract:5];
  NSLog(@"value is now %i ..", [calculator value]);

  // looping
  {
    int i;
    for (i = 0; i < 30; i++) {
      [calculator add:i];
    }
    for (i = 0; i < 30; i++) {
      if (i == 15) sleep(1); // let other clients connect ..
      [calculator subtract:i];
    }
  }

  { // send client-proxy
    NSMutableString *string = [NSMutableString stringWithCapacity:256];

    [string appendString:@"MyString .."];
    NSLog(@"string is now '%s'", [string cString]);

    [calculator addValueToMutableString:string];

    NSLog(@"string is now '%s'", [string cString]);
  }

  TRY { // send method without signature ..
    [calculator doof];
  } END_TRY;
  CATCH (ObjcRuntimeException) {
    printf("catched expected exception: %s\n", [[localException name] cString]);
  }
  END_CATCH;

  TRY { // invoke method that throws an exception
    [calculator throwException];
  } END_TRY;
  CATCH (NSException) {
    printf("catched expected exception: %s\n", [[localException name] cString]);
  }
  END_CATCH;

  { // responds to
    BOOL hasDoof;
    BOOL hasAdd;

    NSLog(@"running respondsTo: on %@ ..", calculator);

    hasDoof = [calculator respondsToSelector:@selector(doof)];
    hasAdd  = [calculator respondsToSelector:@selector(add:)];

    NSLog(@"respondsTo:@doof %s", hasDoof ? "yes" : "no");
    NSLog(@"respondsTo:@add: %s", hasAdd  ? "yes" : "no");
  }

  [calculator release]; calculator = nil;

  RELEASE(pool); pool = nil;
}

static void server(id<NGSocketAddress> _address) {
  NGConnection *connection = nil;

  connection = [NGConnection defaultConnection];
  if ([connection registerAddress:_address]) {
    Calculator *calc = [[Calculator alloc] init];

    [connection setRootObject:calc];
  
    //[(id)connection run];
    
    [[NSRunLoop currentRunLoop] run];

    RELEASE(calc);
  }
  else {
    NSLog(@"could not register connection ..");
  }
}

NGOptionDescription options[] = {
  { @"-s", @"--server", NGRegisterOption, 0, NULL, @"run as server" },
  { @"-p", @"--port",   NGRegisterOption, 1, "i",  @"ORB port" },
  { @"-pp", @"--path-port", NGRegisterOption, 1, "s",  @"ORB path port" },
  { @"-ad",@"--no-autorelease-debug",
    NGRegisterOption, 0, NULL, @"disable autorelease debugging" },
  { @"-?", @"--help",   NGHelpOption,     0, NULL, @"show help page"  },
  NGLogOption,
  NGLastOption
};

NGMainOpt(options) {
  NGInternetSocketAddress *address = nil;
  NGInitTextStdio();

  address = NGSocketAddressFromString([[NSUserDefaults standardUserDefaults]
                                                       stringForKey:@"port"]);
  [NSAutoreleasePool enableDoubleReleaseCheck:![tool isFlagOptionSet:@"-ad"]];

  if ([tool isFlagOptionSet:@"-s"]) {
    [NGTextOut writeFormat:@"starting server at %@ ..\n", address];
    server(address);
  }
  else {
    [NGTextOut writeFormat:@"connecting client to %@ ..\n", address];
    client(address);
  }
}
NGEndMain;

LINK_NGExtensions;
LINK_NGStreams;
LINK_NGNet;
LINK_DO;
