/* 
   echod.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: echod.m,v 1.3 1998/12/18 09:16:05 helge Exp $

#import <Foundation/Foundation.h>
#import <NGExtensions/NGExtensions.h>
#import <NGStreams/NGStreams.h>
#import <NGNet/NGNet.h>

NGOptionDescription options[] = {
  { @"-p", @"--port",    NGRegisterOption, 1, "i",  @"port to bind" },
  { @"-t", @"--threads", NGRegisterOption, 0, NULL, @"run multithreaded" },
  { @"-?", @"--help", NGHelpOption,     0, NULL, @"show help page" },
  NGLastOption
};

@interface ConnectionHandler : NSObject

- (void)handleConnection:(NGActiveSocket *)_socket;

@end

NGMainOpt(options) {
  int port = [tool intValueOfOption:@"-p"];

  NGInitTextStdio();

  [NGTextOut writeFormat:@"time():         %ds\n", time(NULL)];
  [NGTextOut writeFormat:@"NSDate:         %8.0fs\n",
               [[NSDate date] timeIntervalSince1970]];
  [NGTextOut writeFormat:@"NSCalendarDate: %8.0fs\n",
               [[NSCalendarDate date] timeIntervalSince1970]];

  {
    NSCalendarDate *d = nil;
    NSCalendarDate *s, *e;

    d = [NSCalendarDate calendarDate];
    s = [d beginOfDay];
    e = [d endOfDay];
    
    [NGTextOut writeString:@"MET:\n"];
    [NGTextOut writeFormat:@"  %@\n", d];
    [NGTextOut writeFormat:@"  %@\n", s];
    [NGTextOut writeFormat:@"  %@\n", e];

    [d setTimeZone:[NSTimeZone timeZoneWithName:@"GMT"]];
    [s setTimeZone:[NSTimeZone timeZoneWithName:@"GMT"]];
    [e setTimeZone:[NSTimeZone timeZoneWithName:@"GMT"]];
    [NGTextOut writeString:@"GMT:\n"];
    [NGTextOut writeFormat:@"  %@\n", d];
    [NGTextOut writeFormat:@"  %@\n", s];
    [NGTextOut writeFormat:@"  %@\n", e];
    s = [d beginOfDay];
    e = [d endOfDay];
    [NGTextOut writeFormat:@"  %@\n", s];
    [NGTextOut writeFormat:@"  %@\n", e];

    d = [NSCalendarDate date];
    [NGTextOut writeFormat:@"Year: %@\n", [d descriptionWithCalendarFormat:@"%Y %y"]];
    d = [d nextYear];
    [NGTextOut writeFormat:@"Year: %@\n", [d descriptionWithCalendarFormat:@"%Y %y"]];
    d = [d nextYear];
    [NGTextOut writeFormat:@"Year: %@\n", [d descriptionWithCalendarFormat:@"%Y %y"]];
    d = [d nextYear];
    [NGTextOut writeFormat:@"Year: %@\n", [d descriptionWithCalendarFormat:@"%Y %y"]];

    d = [NSCalendarDate dateWithString:@"11:00" calendarFormat:@"%H:%M"];
    [NGTextOut writeFormat:@"Year: %@\n", [d descriptionWithCalendarFormat:@"%Y %y"]];
    d = [NSCalendarDate dateWithString:@"11:01 0" calendarFormat:@"%H:%M %Y"];
    [NGTextOut writeFormat:@"Year: %@\n", [d descriptionWithCalendarFormat:@"%Y %y"]];
    d = [NSCalendarDate dateWithString:@"11:02 1" calendarFormat:@"%H:%M %Y"];
    [NGTextOut writeFormat:@"Year: %@\n", [d descriptionWithCalendarFormat:@"%Y %y"]];
  }

  if (port == 0) {
    [NGTextErr writeString:@"no port set ..\n"];
    [tool exitWithCode:20];
  }

  TRY {
    NGInternetSocketAddress *socketAddress = nil;
    NGPassiveSocket         *socket        = nil;
    
    socketAddress = [NGInternetSocketAddress wildcardAddressWithPort:port];
    socket        = [NGPassiveSocket socketBoundToAddress:socketAddress];

    [socket listenWithBacklog:10];

    while (YES) {
      NGActiveSocket *connection;
      NSAutoreleasePool *pool = [NSAutoreleasePool new];

      NSLog(@"waiting for connection ..");
      connection = [socket accept];

      NSLog(@"accepted connection %@", connection);

      if ([tool isFlagOptionSet:@"-t"]) {
        [NSThread detachNewThreadSelector:@selector(handleConnection:)
                  toTarget:AUTORELEASE([[ConnectionHandler alloc] init])
                  withObject:connection];
      }
      else {
        ConnectionHandler *obj = [[ConnectionHandler alloc] init];
        [obj handleConnection:connection];
        [obj release];
      }

      RELEASE(pool);
    }
  } END_TRY;
  CATCH (NGCouldNotBindSocketException) {
    fprintf(stderr, "address already in use.\n");
    [tool exitWithCode:20];
  }
  END_CATCH;
}
NGEndMain;

@implementation ConnectionHandler

- (void)handleConnection:(NGActiveSocket *)_socket {
  NGCTextStream *stream = [NGCTextStream textStreamWithSource:_socket];
  
  TRY {
    while (YES) {
      NSString *line = [stream readLineAsString];
      fprintf(stdout, "[0x%08X]=> '%s'\n", (unsigned)self, [line cString]);
    }
  } END_TRY;
  CATCH (NGEndOfStreamException) {
    fprintf(stderr, "[0x%08X]=> connection was closed.\n", (unsigned)self);
  }
  CATCH (NSException) {
    fprintf(stderr, "[0x%08X]=> catched %s\n", (unsigned)self,
            [[localException description] cString]);
  }
  END_CATCH;
}

@end

LINK_NGExtensions;
LINK_NGStreams;
LINK_NGNet;
