/* 
   proplist.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: proplist.m,v 1.4 1998/10/19 15:04:55 helge Exp $

#import <NGExtensions/NGPropertyListParser.h>
#import "common.h"

#define COUNT 3000

NGOptionDescription options[] = {
  { @"-d", @"--no-debug", NGRegisterOption, 0, NULL, @"disable double-release chk" },
  { @"-p", @"--propfile", NGRegisterOption, 1, "s",  @"property file to parse"  },
  { @"-s", @"--strfile",  NGRegisterOption, 1, "s",  @"strings file to parse"  },
  { @"-?", @"--help",     NGHelpOption,     0, NULL, @"show help page" },
  NGLogOption,
  NGLastOption
};

NGMainOpt(options) {
  NGInitTextStdio();

  if ([tool isFlagOptionSet:@"-d"])
    [NSAutoreleasePool enableDoubleReleaseCheck:NO];

  if ([tool isOptionSet:@"-s"]) {
    NSString     *path = [tool valueOfOption:@"-s"];
    NSDictionary *pl   = nil;

    pl = NGParseStringsFromFile(path);
    NSLog(@"parsed strings from file %@:\n%@", path, pl);

    pl = NGParseStringsFromData([NSData dataWithContentsOfFile:path]);
    NSLog(@"parsed strings from file %@:\n%@", path, pl);
  }
  else if ([tool isOptionSet:@"-p"]) {
    NSString *path = [tool valueOfOption:@"-p"];
    NSString *string;
    NSData   *data;
    id pl = nil, fd = nil;

    pl = NGParsePropertyListFromFile(path);

    data   = [NSData dataWithContentsOfFile:path];
    string = [NSString stringWithCString:[data bytes] length:[data length]];

    pl = NGParsePropertyListFromData(data);

    if (pl) {
      struct timeval tv;
      NSTimeInterval t;
      
      NSLog(@"parsed property list (isequal=%s, class=%@):\n%@",
            [pl isEqual:fd] ? "yes" : "no", NSStringFromClass([pl class]), pl);

      if (![pl isEqual:fd] && (fd != nil)) {
        NSLog(@"fd parsed property list (class=%@):\n%@",
              NSStringFromClass([fd class]), fd);
      }
      
      // ********************

      NSLog(@"now timing ..");

      gettimeofday(&tv, NULL);
      t = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0);
      {
        int cnt;
        id pla[COUNT];
        for (cnt = 0; cnt < COUNT; cnt++) {
          pla[cnt] = NGParsePropertyListFromData(data);
        }
      }
      gettimeofday(&tv, NULL);
      t = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0) - t;

      NSLog(@"my took %4.3fs.", t < 0.0 ? -1.0 : t);

      // ********************

      NSLog(@"now timing mmap ..");

      gettimeofday(&tv, NULL);
      t = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0);
      {
        int cnt;
        id pla[COUNT];
        
        for (cnt = 0; cnt < COUNT; cnt++)
          pla[cnt] = NGParsePropertyListFromFile(path);
      }
      gettimeofday(&tv, NULL);
      t = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0) - t;

      NSLog(@"mmap took %4.3fs.", t < 0.0 ? -1.0 : t);

      // ********************

      if (fd) {
        NSLog(@"now timing fd-from file ..");

        gettimeofday(&tv, NULL);
        t = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0);
        {
          int cnt;
          id pla[COUNT];
        
          for (cnt = 0; cnt < COUNT; cnt++)
            pla[cnt] = [NSDictionary dictionaryWithContentsOfFile:path];
        }
        gettimeofday(&tv, NULL);
        t = (double)tv.tv_sec + ((double)tv.tv_usec / 1000000.0) - t;

        NSLog(@"fd-file took %4.3fs.", t < 0.0 ? -1.0 : t);
      }
    }
    else {
      NSLog(@"parsing of property list in file %@ failed ..",
            [tool valueOfOption:@"-f"]);
    }
  }
  else
    NSLog(@"you must specify a file ..");
}
NGEndMain;

LINK_NGExtensions;
LINK_NGStreams;
