/* 
   reflection.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: reflection.m,v 1.2 1998/10/02 11:46:26 helge Exp $

#import <Foundation/exceptions/GeneralExceptions.h>
#import <NGExtensions/NGExtensions.h>
#import <NGStreams/NGStreams.h>
#import <NGReflection/NGReflection.h>

NGOptionDescription options[] = {
  { @"-c", @"--class",      NGRegisterOption, 1, "s",  @"class to show"   },
  { @"-m", @"--metaclass",  NGRegisterOption, 0, NULL, @"show metaclass"  },
  { @"-s", @"--subclasses", NGRegisterOption, 0, NULL, @"show subclasses" },
  { @"-?", @"--help",       NGHelpOption,     0, NULL, @"show help page"  },
  NGLogOption,
  NGLastOption
};

NGMainOpt(options) {
  NGInitTextStdio();
  {
    NGObjCClass *classInfo = nil;

    classInfo = [NGObjCClass infoForClassNamed:[tool valueOfOption:@"-c"]];
    if (classInfo == nil) {
      [NGTextErr writeFormat:@"did not find class %@.\n",
                   [tool valueOfOption:@"-c"]];
      [NGTextErr flush];
      [tool exitWithCode:20];
    }

    if ([tool isFlagOptionSet:@"-m"])
      classInfo = [classInfo metaClassInfo];

    if (classInfo == nil) {
      [NGTextErr writeFormat:@"did not find meta class of class %@.\n",
                   [tool valueOfOption:@"-c"]];
      [NGTextErr flush];
      [tool exitWithCode:20];
    }

    [NGTextOut writeString:[classInfo description]];
    [NGTextOut writeNewline];
    [NGTextOut flush];

    [NGTextOut writeString:[[classInfo metaClassInfo] description]];
    [NGTextOut writeNewline];
    [NGTextOut flush];

    [NGTextOut writeString:[[classInfo propertyList] description]];
    [NGTextOut writeNewline];
    [NGTextOut flush];
  }
}
NGEndMain;

LINK_NGExtensions;
LINK_NGStreams;
