/* 
   udp.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: udp.m,v 1.2 1998/10/02 11:46:28 helge Exp $

#include <unistd.h>
#include <stdio.h>
#import "common.h"

static int  packetSize = 256;
static BOOL asServer   = NO;
static int  port       = 10000;

static void runServer(void) {
  NGInternetSocketAddress *address = nil;
  NGDatagramSocket        *socket  = nil;

  address = [NGInternetSocketAddress addressWithPort:port];
  socket  = [[NGDatagramSocket alloc]
                               initWithDomain:[NGInternetSocketDomain domain]];

  [socket setMaxPacketSize:packetSize];

  [socket bindToAddress:address];

  NGLog(@"server %@: start loop ..", socket);
  while (YES) {
    NGDatagramPacket *packet = nil;
    packet = [socket receivePacketWithTimeout:10.0];

    if (packet == nil)
      NGLog(@"server: received timed out ..");
    else {
      NGLog(@"server: value='%s' packet %@", [[packet data] bytes], packet);

      [packet reverseAddresses]; // exchange sender & receiver
      [socket sendPacket:packet];
    }
  }

  RELEASE(socket); socket = nil;
}

static void runClient(void) {
  NGInternetSocketAddress *address = nil;
  NGDatagramSocket        *socket  = nil;
  NGDatagramPacket        *packet  = nil;
  NSString                *value   = nil;

  address = [NGInternetSocketAddress addressWithPort:port];
  socket  = [[NGDatagramSocket alloc]
                               initWithDomain:[NGInternetSocketDomain domain]];
  [socket setMaxPacketSize:packetSize];

  value = [NSString stringWithFormat:@"client %i", getpid()];

  NGLog(@"client %@ ..", socket);

  packet = [[NGDatagramPacket alloc] init];
  [packet setData:[NSData dataWithBytes:"hello" length:6]];
  [packet setReceiver:address];

  NGLog(@"client: send packet %@ ..", packet);
  [socket sendPacket:packet];
  RELEASE(packet); packet = nil;
  
  NGLog(@"client: receive packet ..", packet);
  packet = [socket receivePacketWithTimeout:1.0]; // wait 1s for packet
  NGLog(@"client: value='%s' packet %@.", [[packet data] bytes], packet);
  
  RELEASE(socket); socket = nil;
}

NGOptionDescription options[] = {
  { @"-s",   @"--server", NGRegisterOption, 0, NULL, @"run as server" },
  { @"-p",   @"--port",   NGRegisterOption, 1, "i",  @"UDP port" },
  { @"-?",   @"--help",   NGHelpOption,     0, NULL, @"show help page" },
  NGLogOption,
  NGLastOption
};

NGMainOpt(options) {
  NGInitTextStdio();

  asServer = [tool isFlagOptionSet:@"-s"];
  port     = [tool intValueOfOption:@"-p"];

  if (asServer) runServer();
  else          runClient();
}
NGEndMain;

LINK_NGExtensions;
LINK_NGStreams;
LINK_NGNet;
