/* 
   base64.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: base64.m,v 1.6 1999/03/16 14:21:04 helge Exp $

#import "common.h"

NGOptionDescription options[] = {
  { @"-s",   @"--string", NGRegisterOption, 1, "s",  @"string to encode" },
  { @"-f",   @"--file",   NGRegisterOption, 1, "s",  @"file to encode" },
  { @"-d",   @"--decode", NGRegisterOption, 0, NULL, @"decode instead of encode" },
  { @"-?",   @"--help",   NGHelpOption,     0, NULL, @"show help page" },
  NGLastOption
};

NGMainOpt(options) {
  BOOL doDecode;
  
  NGInitTextStdio();

  doDecode = [tool isFlagOptionSet:@"-d"];

  if ([tool isOptionSet:@"-s"]) {
    NSString *value = [tool valueOfOption:@"-s"];

    value = doDecode
      ? [value stringByDecodingBase64]
      : [value stringByEncodingBase64];

    if (value == nil) {
      NSLog(@"could not encode/decode value !");
      [tool exitWithCode:20];
    }
    
    [NGTextOut writeString:value];
    [NGTextOut writeNewline];
    [NGTextOut flush];
  }
  
  if ([tool isOptionSet:@"-f"]) {
    NSData *data = [NSData dataWithContentsOfFile:[tool valueOfOption:@"-f"]];

    if (data) {
      [NGTextOut writeFormat:@"original=%i bytes\n", [data length]];
      
      data = doDecode
        ? [data dataByDecodingBase64]
        : [data dataByEncodingBase64];

      if (data == nil) {
        NSLog(@"could not encode/decode data !");
        [tool exitWithCode:20];
      }

      [NGTextOut writeString:[NSString stringWithCString:[data bytes]
                                       length:[data length]]];
      [NGTextOut writeNewline];
      [NGTextOut writeFormat:@"copy=%i bytes\n", [data length]];
      [NGTextOut flush];

      {
        NGDataStream   *ds = [NGDataStream streamWithData:data];
        NGBase64Stream *bs = [NGBase64Stream filterWithSource:ds];
        NGCTextStream  *in = [NGCTextStream textStreamWithSource:bs];
        int count = 0;

        {
          NSString *line;

          while ((line = [in readLineAsString])) {
            [NGTextOut writeString:line];
            [NGTextOut writeNewline];
            count++;
          }
        }
        NSLog(@"read %i lines ..", count);
      }
    }
    else {
      [NGTextErr writeFormat:@"base64: could not read file %@\n",
                   [tool valueOfOption:@"-f"]];
      [NGTextErr flush];

      [tool exitWithCode:20];
    }
  }
}
NGEndMain;

LINK_NGExtensions;
LINK_NGStreams;
