/* 
   NGDocGenerator.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: NGDocGenerator.m,v 1.4 1998/10/02 11:48:51 helge Exp $

#import "common.h"
#import "NGDocGenerator.h"

static NSString *AppleDocuPrefix =
  @"http://developer.apple.com/techpubs/rhapsody/System/Library/"
  @"Frameworks/Foundation.framework/Versions/C/Resources/English.lproj/"
  @"Documentation/Reference/ObjC_classic/Classes/";

@interface NGDocGenerator(PrivateMethods)
- (void)_processClass:(NGObjCClass *)_class indent:(int)_level;
@end

@implementation NGDocGenerator

- (id)init {
  if ((self = [super init])) {
    self->titleColor     = @"#AAAAAA";
    self->inheritedColor = @"#444444";
    self->indent         = 2;

    self->processedClasses = [[NSMutableArray allocWithZone:[self zone]]
                                              initWithCapacity:256];

    self->flags.justTrace          = NO;
    self->flags.showPrivateClasses = NO;
    self->flags.showPrivateMethods = NO;
  }
  return self;
}

- (void)dealloc {
  RELEASE(self->dir);              self->dir              = nil;
  RELEASE(self->titleColor);       self->titleColor       = nil;
  RELEASE(self->inheritedColor);   self->inheritedColor   = nil;
  RELEASE(self->processedClasses); self->processedClasses = nil;
  [super dealloc];
}

// accessors

- (void)setDocDirectory:(NGFileUrl *)_dir {
  ASSIGN(self->dir, _dir);
}
- (NGFileUrl *)docDirectory {
  return self->dir;
}

// sort functions

static int sortByOffset(id ivar1, id ivar2, void *context) {
  int o1 = [ivar1 offset];
  int o2 = [ivar2 offset];

  if (o1 == o2)
    return 0;
  else if (o1 < o2)
    return -1;
  else
    return 1;
}
static int sortByClassName(id cls1, id cls2, void *context) {
  return [[cls1 className] compare:[cls2 className]];
}
static int sortByMethodName(id mth1, id mth2, void *context) {
  return [[mth1 methodName] compare:[mth2 methodName]];
}

// generic methods

- (void)_beginTableSection:(NSString *)_title stream:(NGCTextStream *)_out {
  [_out writeString:@"        <TR>\n"];
  [_out writeFormat:@"          <TD BGCOLOR=%@>%@</TD>\n", titleColor, _title];
  [_out writeString:@"        </TR>\n"];
  [_out writeString:@"        <TR><TD>\n"];
}
- (void)_endTableSection:(NGCTextStream *)_out {
  [_out writeString:@"        </TD></TR>\n"];
}

- (void)_genLinkToClass:(NGObjCClass *)_class stream:(NGCTextStream *)_out {
  [_out writeFormat:@"<A HREF=\"%@-%i.html\">",
          [_class className], [_class classVersion]];
  [_out writeString:[_class className]];
  [_out writeString:@"</A>"];
}

static NSString *_genObjCType(NGObjCInstanceVariable *_ivar) {
  if ([_ivar isObjectType]) {
    NSString *objectType = [_ivar objectType];

    if (objectType) {
      Class       cls   = NSClassFromString(objectType);
      NGObjCClass *info = [NGObjCClass infoForClass:cls];

      if (info) {
        NSMutableString *str = [[NSMutableString alloc] initWithCapacity:128];

        [str appendFormat:@"<A HREF=\"%@-%i.html\">%@</A> *",
               [info className],
               [info classVersion],
               [info className]
             ];
      
        return AUTORELEASE(str);
      }
    }
  }
  return [_ivar typeAsObjectiveC];
}

- (void)_genIvarsForClass:(NGObjCClass *)_class stream:(NGCTextStream *)_out {
  id ivars = [[_class instanceVariables] allValues];

  if ([ivars count] > 0) {
    NGObjCInstanceVariable *ivar = nil;

    ivars = [ivars sortedArrayUsingFunction:sortByOffset context:_class];
    ivars = [ivars objectEnumerator];
      
    [self _beginTableSection:@"Instance Variables" stream:_out];
    [_out writeString:
            @"<TABLE BORDER=0 WIDTH=100%>\n"
            @"  <TR>\n"
            @"    <TD WIDTH=30%><I>Name</I></TD>\n"
            @"    <TD WIDTH=40%><I>Type</I></TD>\n"
            @"    <TD WIDTH=10%><I>Offset</I></TD>\n"
            @"    <TD><I>Inherited</I></TD>\n"
            @"  </TR>\n"];

    while ((ivar = [ivars nextObject])) {
      NGObjCClass *inheritedFrom = [ivar variableClass];
      
      if (inheritedFrom == _class)
        inheritedFrom = nil;
        
      [_out writeString:@"  <TR>\n"];
      if (inheritedFrom) {
        [_out writeFormat:
                @"    <TD><FONT COLOR=%@>%@</FONT></TD>\n"
                @"    <TD><FONT COLOR=%@>%@</FONT></TD>\n"
                @"    <TD><FONT COLOR=%@>%i</FONT></TD>\n",
                inheritedColor, [ivar variableName],
                inheritedColor, _genObjCType(ivar),
                inheritedColor, [ivar offset]
        ];
        [_out writeString:@"    <TD>"];
        [self _genLinkToClass:inheritedFrom stream:_out];
        [_out writeString:@"</TD>\n"];
      }
      else {
        [_out writeFormat:
                @"    <TD>%@</TD>\n"
                @"    <TD>%@</TD>\n"
                @"    <TD>%i</TD>\n",
                [ivar variableName],
                _genObjCType(ivar),
                [ivar offset]
        ];
      }
      [_out writeString:@"  </TR>\n"];
    }

    [_out writeString:@"</TABLE>\n"];
    [self _endTableSection:_out];
  }
}

- (void)_genMethods:(id)methods forClass:(NGObjCClass *)_class
  prefix:(NSString *)_prefix stream:(NGCTextStream *)_out {
  
  NGObjCMethod *method = nil;

  [_out writeString:
          @"<TABLE BORDER=0 WIDTH=100%>\n"
          @"  <TR>\n"
          @"    <TD WIDTH=60%><I>Name</I></TD>\n"
          @"    <TD WIDTH=40%><I>Signature</I></TD>\n"
          @"  </TR>\n"];

  while ((method = [methods nextObject])) {
    if (!self->flags.showPrivateMethods && [[method methodName] hasPrefix:@"_"])
      continue;
    
    [_out writeString:@"  <TR>\n"];
    [_out writeFormat:
            @"    <TD>%@%@</TD>\n"
            @"    <TD>%@</TD>\n",
            _prefix, [method methodName],
            [method methodTypes]
          ];
    [_out writeString:@"  </TR>\n"];
  }
    
  [_out writeString:@"</TABLE>\n"];
}

// global methods

- (void)_genDocuForClass:(NGObjCClass *)_class stream:(NGCTextStream *)_out {
  id subclasses = nil;

  if ([_class hasSubclasses])
    subclasses = [_class subclasses];
  
  [_out writeString:@"<HTML>\n  <HEAD><TITLE>"];
  [_out writeString:[_class className]];
  [_out writeString:@"</TITLE></HEAD>\n"];

  [_out writeString:
          @"  <BODY BGCOLOR=#FFFFFF>\n"
          @"    <SMALL>\n"
          @"      <A HREF=\"index.html\">index</A>\n"
          @"    </SMALL>\n"
          @"    <HR>\n"];
  [_out writeFormat:@"    <H3>Class %@</H3>\n", [_class className]];
  [_out writeString:@"    <CENTER>\n"];

  [_out writeString:@"      <TABLE BORDER=0 WIDTH=95%%>\n"];

  [self _beginTableSection:@"Class Info" stream:_out];
  {
    [_out writeString:@"<TABLE BORDER=0 WIDTH=100%>\n"];

    [_out writeString:@"  <TR>\n"];
    [_out writeString:@"    <TD WIDTH=30%><I>Class Name:</I></TD>\n"];
    [_out writeFormat:@"    <TD><B>%@</B></TD>\n", [_class className]];
    [_out writeString:@"  </TR>\n"];

    if (![_class isRootClass]) {
      NGObjCClass *superClass = [_class superClassInfo];
      [_out writeString:@"  <TR>\n"];
      [_out writeString:@"    <TD><I>Super Class:</I></TD>\n"];
      [_out writeString:@"    <TD>\n"];
      [self _genLinkToClass:superClass stream:_out];
      [_out writeString:@"    </TD>\n"];
      [_out writeString:@"  </TR>\n"];
    }
    [_out writeString:@"  <TR>\n"];
    [_out writeString:@"    <TD><I>Subclasses:</I></TD>\n"];
    if ([subclasses count] > 0)
      [_out writeFormat:@"    <TD>%i direct subclasses</TD>\n", [subclasses count]];
    else
      [_out writeString:@"    <TD>no subclasses</TD>\n"];
    [_out writeString:@"  </TR>\n"];

    [_out writeString:@"  <TR>\n"];
    [_out writeString:@"    <TD><I>Version:</I></TD>\n"];
    [_out writeFormat:@"    <TD>%i</TD>\n", [_class classVersion]];
    [_out writeString:@"  </TR>\n"];

    [_out writeString:@"  <TR>\n"];
    [_out writeString:@"    <TD><I>Number:</I></TD>\n"];
    [_out writeFormat:@"    <TD>%i</TD>\n", [_class classNumber]];
    [_out writeString:@"  </TR>\n"];

    [_out writeString:@"  <TR>\n"];
    [_out writeString:@"    <TD><I>Instance Size:</I></TD>\n"];
    [_out writeFormat:@"    <TD>%i bytes</TD>\n", [_class instanceSize]];
    [_out writeString:@"  </TR>\n"];

    if ([[_class className] hasPrefix:@"NS"] &&
        ![[_class className] hasPrefix:@"NSConcrete"]) {
      [_out writeString:@"  <TR>\n"];
      [_out writeString:@"    <TD><I>Documentation:</I></TD>\n"];
      [_out writeFormat:@"    <TD><A HREF=\"%@%@.html\">%@</A></TD>\n",
              AppleDocuPrefix, [_class className], [_class className]];
      [_out writeString:@"  </TR>\n"];
    }
    
    [_out writeString:@"</TABLE>"];
  }
  [self _endTableSection:_out];

  // instance variables
  [self _genIvarsForClass:_class stream:_out];

  // instance methods
  {
    id methods = [_class methods];
    if ([methods count] > 0) {
      methods = [methods sortedArrayUsingFunction:sortByMethodName context:_class];
      methods = [methods objectEnumerator];

      [self _beginTableSection:@"Instance Methods" stream:_out];
      [self _genMethods:methods forClass:_class prefix:@"- " stream:_out];
      [self _endTableSection:_out];
    }
  }

  // class methods
  {
    id methods = [_class classMethods];
    if ([methods count] > 0) {
      methods = [methods sortedArrayUsingFunction:sortByMethodName context:_class];
      methods = [methods objectEnumerator];

      [self _beginTableSection:@"Class Methods" stream:_out];
      [self _genMethods:methods forClass:_class prefix:@"+ " stream:_out];
      [self _endTableSection:_out];
    }
  }
  
  // direct subclasses
  if ([subclasses count] > 0) {
    NGObjCClass *subclass = nil;
    BOOL        didWriteHead = NO;
      
    subclasses = [subclasses sortedArrayUsingFunction:sortByClassName
                             context:_class];
    subclasses = [subclasses objectEnumerator];
      
    while ((subclass = [subclasses nextObject])) {
      if (!self->flags.showPrivateClasses && [[subclass className] hasPrefix:@"_"])
        continue;

      if (!didWriteHead) {
        [self _beginTableSection:@"Direct Subclasses" stream:_out];
        didWriteHead = YES;
      }
      
      [_out writeString:@"  <LI>"];
      [self _genLinkToClass:subclass stream:_out];
      [_out writeString:@"</LI>\n"];
    }
    if (didWriteHead)
      [self _endTableSection:_out];
  }
  
  [_out writeString:@"      </TABLE>\n"];
  
  [_out writeString:
          @"    </CENTER>\n"
          @"    <HR>\n"
          @"    <SMALL>file generated on "];
  [_out writeString:[[NSCalendarDate date] description]];
  [_out writeString:@", "];
  [_out writeString:
          @"docgen by Helge He&szlig;"
          @" [<A HREF=\"mailto:hh@mdlink.de\">hh@mdlink.de</A>]</SMALL>"
          @"  </BODY>\n"
          @"</HTML>\n"];
}

- (void)_genDocuForSubclassesOfClass:(NGObjCClass *)_superClass level:(int)_level {
  NSAutoreleasePool *pool = [NSAutoreleasePool new];
  id           subclasses = [_superClass subclasses];
  NGObjCClass  *class      = nil;

  if ([subclasses count] > 0) {
    subclasses = [subclasses sortedArrayUsingFunction:sortByClassName
                             context:_superClass];
    subclasses = [subclasses objectEnumerator];

    while ((class = [subclasses nextObject])) {
      if (!self->flags.showPrivateClasses && [[class className] hasPrefix:@"_"])
        continue;

      [self _processClass:class indent:(_level + self->indent)];
    }
  }
  RELEASE(pool);
}

- (void)_genIndexFile:(id)_classes {
  if (!self->flags.justTrace) {
    NGFileUrl        *url = nil;
    NGFileStream     *fs  = nil;
    NGBufferedStream *bs  = nil;
    NGCTextStream    *ts  = nil;

    url = [NGFileUrl fileUrlWithString:
                       [NSString stringWithFormat:@"%@/index.html", [dir path]]];
    
    _classes = [_classes sortedArrayUsingFunction:sortByClassName context:_classes];
    _classes = [_classes objectEnumerator];

    fs = [[NGFileStream alloc] initWithFileUrl:url];
    TRY {
      [fs openInMode:NGFileWriteOnly];
    } END_TRY;
    CATCH (NGCouldNotOpenStreamException) {
      RELEASE(fs); fs = nil;
      NGCLog("%@", localException);
    }
    END_CATCH;

    if (fs) {
      NGObjCClass *class = nil;
      
      bs = [[NGBufferedStream alloc] initWithSource:fs];
      ts = [[NGCTextStream    alloc] initWithSource:bs];

      [ts writeString:@"<HTML>\n  <HEAD><TITLE>Class Index</TITLE></HEAD>\n"];

      [ts writeString:
            @"  <BODY BGCOLOR=#FFFFFF>\n"
            @"    <H3>Class Index</H3>\n"
            @"    <CENTER>\n"
            @"      <TABLE BORDER=0 WIDTH=95%>\n"
            @"        <TR>\n"
            @"          <TD BGCOLOR=#AAAAAA WIDTH=30%><I>Class</I></TD>\n"
            @"          <TD BGCOLOR=#AAAAAA WIDTH=30%><I>Super Class</I></TD>\n"
            @"          <TD BGCOLOR=#AAAAAA WIDTH=5%><I>Version</I></TD>\n"
            @"        </TR>\n"];
      
      while ((class = [_classes nextObject])) {
        [ts writeString:@"        <TR>\n"];

        [ts writeString:@"          <TD>"];
        [self _genLinkToClass:class stream:ts];
        
        if ([[class className] hasPrefix:@"NS"] &&
            ![[class className] hasPrefix:@"NSConcrete"]) {
          [ts writeFormat:@" <A HREF=\"%@%@.html\">[docu]</A>",
                AppleDocuPrefix, [class className]];
        }
        [ts writeString:@"</TD>\n"];

        if ([class isRootClass])
          [ts writeString:@"          <TD>root class</TD>\n"];
        else {
          [ts writeString:@"          <TD>"];
          [self _genLinkToClass:[class superClassInfo] stream:ts];
          [ts writeString:@"</TD>\n"];
        }

        [ts writeFormat:@"          <TD>%i</TD>\n", [class classVersion]];
        [ts writeString:@"        </TR>\n"];
      }
      
      [ts writeString:
            @"      </TABLE>\n"
            @"    </CENTER>\n"
            @"    <HR>\n"
            @"    <SMALL>file generated on "];
      [ts writeString:[[NSCalendarDate date] description]];
      [ts writeString:
            @", docgen by Helge He&szlig;"
            @" [<A HREF=\"mailto:hh@mdlink.de\">hh@mdlink.de</A>]</SMALL>"
            @"  </BODY>\n"
            @"</HTML>\n"];

      [ts flush];
      [ts close];
      RELEASE(ts); ts = nil;
      RELEASE(bs); bs = nil;
      RELEASE(fs); fs = nil;
    }
  }
}

- (void)_processClass:(NGObjCClass *)_class indent:(int)_level {
  NGFileUrl *url = nil;

  url = [NGFileUrl fileUrlWithString:
                     [NSString stringWithFormat:@"%@/%@-%i.html",
                                 [dir path],
                                 [_class className],
                                 [_class classVersion]]];
  if ([url doesResourceExist]) {
    [NGTextOut writeFormat:@"reference for class %@ already exists, skipping.\n",
                 [_class className]];
  }
  else {
    int cnt;

    [processedClasses addObject:_class];

    for (cnt = 0; cnt < _level; cnt++)
      [NGTextOut writeString:@" "];
    [NGTextOut writeFormat:@"%@ (%@)\n", [_class className], url];

    if (!self->flags.justTrace) {
      NGFileStream     *fs  = nil;
      NGBufferedStream *bs  = nil;
      NGCTextStream    *ts  = nil;
      
      fs = [[NGFileStream alloc] initWithFileUrl:url];
      TRY {
        [fs openInMode:NGFileWriteOnly];
      } END_TRY;
      CATCH (NGCouldNotOpenStreamException) {
        RELEASE(fs); fs = nil;
        NGCLog("%@", localException);
      }
      END_CATCH;

      if (fs) {
        bs = [[NGBufferedStream alloc] initWithSource:fs];
        ts = [[NGCTextStream    alloc] initWithSource:bs];

        [self _genDocuForClass:_class stream:ts];

        [ts flush];
        [ts close];
        RELEASE(ts); ts = nil;
        RELEASE(bs); bs = nil;
        RELEASE(fs); fs = nil;
      }
    }
  }
  [self _genDocuForSubclassesOfClass:_class level:_level];
}

// root

- (void)processClass:(NGObjCClass *)_class {
  [self->processedClasses removeAllObjects];

  [self _processClass:_class indent:indent];
  
  [NGTextOut writeFormat:@"processed %i classes, generating index file ..\n",
               [processedClasses count]];

  [self _genIndexFile:processedClasses];
  [self->processedClasses removeAllObjects];
}

@end
