/* 
   docgen.m

   Copyright (C) 1998 MDlink online service center, Helge Hess
   All rights reserved.

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the MDlink Object Framework 2 (MOF2)

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.

   We disclaim all warranties with regard to this software, including all
   implied warranties of merchantability and fitness, in no event shall
   we be liable for any special, indirect or consequential damages or any
   damages whatsoever resulting from loss of use, data or profits, whether in
   an action of contract, negligence or other tortious action, arising out of
   or in connection with the use or performance of this software.
*/
// $Id: docgen.m,v 1.9 1998/12/28 08:42:02 helge Exp $

#import "common.h"
#import "NGDocGenerator.h"

NGOptionDescription options[] = {
  { @"-d",   @"--directory", NGRegisterOption, 1, "s",  @"target directory"   },
  { @"-c",   @"--class",     NGRegisterOption, 1, "s",  @"root class"         },
  { @"-t",   @"--trace",     NGRegisterOption, 0, NULL, @"only trace classes" },
  { @"-pc",  @"--private-classes",NGRegisterOption,0,NULL,@"show private classes" },
  { @"-pm",  @"--private-methods",NGRegisterOption,0,NULL,@"show private methods" },
  { @"-?",   @"--help",      NGHelpOption,     0, NULL, @"show help page"     },
  NGLogOption,
  NGLastOption
};

NGMainOpt(options) {
  NGObjCClass *root = nil;
  NGFileUrl   *dir               = nil;
  BOOL        justTrace          = NO;
  BOOL        showPrivateClasses = NO;
  BOOL        showPrivateMethods = NO;
  
  NGInitTextStdio();

  justTrace          = [tool isFlagOptionSet:@"-t"];
  showPrivateClasses = [tool isFlagOptionSet:@"-pc"];
  showPrivateMethods = [tool isFlagOptionSet:@"-pm"];

  if (!justTrace) {
    if (![tool isOptionSet:@"-d"]) {
      [NGTextErr writeString:@"set '-d' option ..\n"];
      [tool exitWithCode:20];
    }
    else {
      dir = [NGFileUrl fileUrlWithPath:[tool valueOfOption:@"-d"]];
    
      if ([dir doesResourceExist]) {
        if (![dir isCollection]) {
          [NGTextErr writeFormat:@"'%@' is a file, not a directory !\n", dir];
          [tool exitWithCode:20];
        }
      }
      else {
        [NGTextErr writeFormat:@"directory '%@' does not exist !\n", dir];
        [tool exitWithCode:20];
      }
    }
  }

  [NSAutoreleasePool enableDoubleReleaseCheck:NO];

  if (![tool isOptionSet:@"-c"])
    root = [NGObjCClass infoForClassNamed:@"NSObject"];
  else {
    root = [NGObjCClass infoForClassNamed:[tool valueOfOption:@"-c"]];
    if (root == nil) {
      [NGTextErr writeFormat:@"did not find class named '%@' !\n",
                   [tool valueOfOption:@"-c"]];
      [tool exitWithCode:20];
    }
  }

  [NGTextOut writeFormat:@"generating reference for class %@ in directory %@ ..\n",
               [root className], dir];
  {
    NGDocGenerator *gen = [[NGDocGenerator alloc] init];

    [gen setDocDirectory:dir];
    [gen processClass:root];

    RELEASE(gen); gen = nil;
  }
  [NGTextOut writeString:@"generation done.\n"];
}
NGEndMain;

LINK_NGExtensions;
LINK_NGStreams;
LINK_NGReflection;

#import <NGNet/NGNet.h>

LINK_NGNet; // just to ref ..
